@@ -46,6 +46,82 @@
 #define AOUT_AUDIOTRACK_JAVA 1
 #define AOUT_OPENSLES        2
 
+static jint getInt(JNIEnv *env, jobject thiz, const char* field) {
+    jclass clazz = (*env)->GetObjectClass(env, thiz);
+    jfieldID fieldMP = (*env)->GetFieldID(env, clazz,
+                                          field, "I");
+    return (*env)->GetIntField(env, thiz, fieldMP);
+}
+static void setInt(JNIEnv *env, jobject item, const char* field, jint value) {
+    jclass cls;
+    jfieldID fieldId;
+
+    /* Get a reference to item's class */
+    cls = (*env)->GetObjectClass(env, item);
+
+    /* Look for the instance field s in cls */
+    fieldId = (*env)->GetFieldID(env, cls, field, "I");
+    if (fieldId == NULL)
+        return;
+
+    (*env)->SetIntField(env, item, fieldId, value);
+}
+
+static jlong getLong(JNIEnv *env, jobject thiz, const char* field) {
+    jclass clazz = (*env)->GetObjectClass(env, thiz);
+    jfieldID fieldMP = (*env)->GetFieldID(env, clazz,
+                                          field, "J");
+    return (*env)->GetLongField(env, thiz, fieldMP);
+}
+static void setLong(JNIEnv *env, jobject item, const char* field, jlong value) {
+    jclass cls;
+    jfieldID fieldId;
+
+    /* Get a reference to item's class */
+    cls = (*env)->GetObjectClass(env, item);
+
+    /* Look for the instance field s in cls */
+    fieldId = (*env)->GetFieldID(env, cls, field, "J");
+    if (fieldId == NULL)
+        return;
+
+    (*env)->SetLongField(env, item, fieldId, value);
+}
+
+static void setFloat(JNIEnv *env, jobject item, const char* field, jfloat value) {
+    jclass cls;
+    jfieldID fieldId;
+
+    /* Get a reference to item's class */
+    cls = (*env)->GetObjectClass(env, item);
+
+    /* Look for the instance field s in cls */
+    fieldId = (*env)->GetFieldID(env, cls, field, "F");
+    if (fieldId == NULL)
+        return;
+
+    (*env)->SetFloatField(env, item, fieldId, value);
+}
+static void setString(JNIEnv *env, jobject item, const char* field, const char* text) {
+    jclass cls;
+    jfieldID fieldId;
+    jstring jstr;
+
+    /* Get a reference to item's class */
+    cls = (*env)->GetObjectClass(env, item);
+
+    /* Look for the instance field s in cls */
+    fieldId = (*env)->GetFieldID(env, cls, field, "Ljava/lang/String;");
+    if (fieldId == NULL)
+        return;
+
+    /* Create a new string and overwrite the instance field */
+    jstr = (*env)->NewStringUTF(env, text);
+    if (jstr == NULL)
+        return;
+    (*env)->SetObjectField(env, item, fieldId, jstr);
+}
+
 struct length_change_monitor {
     pthread_mutex_t doneMutex;
     pthread_cond_t doneCondVar;
@@ -61,7 +137,7 @@ static void length_changed_callback(const libvlc_event_t *ev, void *data)
     pthread_mutex_unlock(&monitor->doneMutex);
 }
 
-libvlc_media_t *new_media(jint instance, JNIEnv *env, jobject thiz, jstring fileLocation, bool noOmx, bool noVideo)
+libvlc_media_t *new_media(jlong instance, JNIEnv *env, jobject thiz, jstring fileLocation, bool noOmx, bool noVideo)
 {
     libvlc_instance_t *libvlc = (libvlc_instance_t*)instance;
     jboolean isCopy;
@@ -94,30 +170,37 @@ libvlc_media_t *new_media(jint instance, JNIEnv *env, jobject thiz, jstring file
     return p_md;
 }
 
+static libvlc_media_list_t *getMediaList(JNIEnv *env, jobject thiz)
+{
+    return (libvlc_media_list_t*)getLong(env, thiz, "mMediaListInstance");
+}
+
 static libvlc_media_player_t *getMediaPlayer(JNIEnv *env, jobject thiz)
 {
-    jclass clazz = (*env)->GetObjectClass(env, thiz);
-    jfieldID fieldMP = (*env)->GetFieldID(env, clazz,
-                                          "mMediaPlayerInstance", "I");
-    return (libvlc_media_player_t*)(*env)->GetIntField(env, thiz, fieldMP);
+    return (libvlc_media_player_t*)getLong(env, thiz, "mInternalMediaPlayerInstance");
+}
+
+static libvlc_media_list_player_t *getMediaListPlayer(JNIEnv *env, jobject thiz)
+{
+    return (libvlc_media_list_player_t*)getLong(env, thiz, "mMediaListPlayerInstance");
 }
 
 static void unsetMediaPlayer(JNIEnv *env, jobject thiz)
 {
-    jclass clazz = (*env)->GetObjectClass(env, thiz);
-    jfieldID fieldMP = (*env)->GetFieldID(env, clazz,
-                                          "mMediaPlayerInstance", "I");
-    (*env)->SetIntField(env, thiz, fieldMP, 0);
+    setLong(env, thiz, "mInternalMediaPlayerInstance", (jlong)0);
 }
 
 static void releaseMediaPlayer(JNIEnv *env, jobject thiz)
 {
-    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
-    if (mp)
+    libvlc_media_list_player_t* p_mlp = getMediaListPlayer(env, thiz);
+    if (p_mlp)
     {
-        libvlc_media_player_stop(mp);
-        libvlc_media_player_release(mp);
+        libvlc_media_list_player_stop(p_mlp);
+        libvlc_media_list_player_release(p_mlp);
+        /* libvlc_media_list_player_release frees the media player, so
+         * we don't free it ourselves. */
         unsetMediaPlayer(env, thiz);
+        setLong(env, thiz, "mInternalMediaPlayerInstance", 0);
     }
 }
 
@@ -142,7 +225,7 @@ void jni_UnlockAndroidSurface() {
     pthread_mutex_unlock(&vout_android_lock);
 }
 
-void jni_SetAndroidSurfaceSize(int width, int height)
+void jni_SetAndroidSurfaceSize(int width, int height, int sar_num, int sar_den)
 {
     if (vout_android_gui == NULL)
         return;
@@ -151,9 +234,9 @@ void jni_SetAndroidSurfaceSize(int width, int height)
 
     (*myVm)->AttachCurrentThread (myVm, &p_env, NULL);
     jclass cls = (*p_env)->GetObjectClass (p_env, vout_android_gui);
-    jmethodID methodId = (*p_env)->GetMethodID (p_env, cls, "setSurfaceSize", "(II)V");
+    jmethodID methodId = (*p_env)->GetMethodID (p_env, cls, "setSurfaceSize", "(IIII)V");
 
-    (*p_env)->CallVoidMethod (p_env, vout_android_gui, methodId, width, height);
+    (*p_env)->CallVoidMethod (p_env, vout_android_gui, methodId, width, height, sar_num, sar_den);
 
     (*p_env)->DeleteLocalRef(p_env, cls);
     (*myVm)->DetachCurrentThread (myVm);
@@ -164,11 +247,6 @@ static void vlc_event_callback(const libvlc_event_t *ev, void *data)
     JNIEnv *env;
     JavaVM *myVm = data;
 
-    int ev_opt_data = 0;
-    if(ev->type == libvlc_MediaPlayerVout) {
-        /* For determining the vout/ES track change */
-        ev_opt_data = ev->u.media_player_vout.new_count;
-    }
     bool isAttached = false;
 
     if (eventManagerInstance == NULL)
@@ -184,6 +262,48 @@ static void vlc_event_callback(const libvlc_event_t *ev, void *data)
         isAttached = true;
     }
 
+    /* Creating the bundle in C allows us to subscribe to more events
+     * and get better flexibility for each event. For example, we can
+     * have totally different types of data for each event, instead of,
+     * for example, only an integer and/or string.
+     */
+    jclass clsBundle = (*env)->FindClass(env, "android/os/Bundle");
+    jmethodID clsCtor = (*env)->GetMethodID(env, clsBundle, "<init>", "()V" );
+    jobject bundle = (*env)->NewObject(env, clsBundle, clsCtor);
+
+    jmethodID putInt = (*env)->GetMethodID(env, clsBundle, "putInt", "(Ljava/lang/String;I)V" );
+    jmethodID putString = (*env)->GetMethodID(env, clsBundle, "putString", "(Ljava/lang/String;Ljava/lang/String;)V" );
+
+    if(ev->type == libvlc_MediaPlayerVout) {
+        /* For determining the vout/ES track change */
+        jstring sData = (*env)->NewStringUTF(env, "data");
+        (*env)->CallVoidMethod(env, bundle, putInt, sData, ev->u.media_player_vout.new_count);
+        (*env)->DeleteLocalRef(env, sData);
+    } else if(ev->type == libvlc_MediaListItemAdded ||
+              ev->type == libvlc_MediaListItemDeleted ) {
+        jstring item_uri = (*env)->NewStringUTF(env, "item_uri");
+        jstring item_index = (*env)->NewStringUTF(env, "item_index");
+        char* mrl = libvlc_media_get_mrl(
+            ev->type == libvlc_MediaListItemAdded ?
+            ev->u.media_list_item_added.item :
+            ev->u.media_list_item_deleted.item
+            );
+        jstring item_uri_value = (*env)->NewStringUTF(env, mrl);
+        jint item_index_value;
+        if(ev->type == libvlc_MediaListItemAdded)
+            item_index_value = ev->u.media_list_item_added.index;
+        else
+            item_index_value = ev->u.media_list_item_deleted.index;
+
+        (*env)->CallVoidMethod(env, bundle, putString, item_uri, item_uri_value);
+        (*env)->CallVoidMethod(env, bundle, putInt, item_index, item_index_value);
+
+        (*env)->DeleteLocalRef(env, item_uri);
+        (*env)->DeleteLocalRef(env, item_uri_value);
+        (*env)->DeleteLocalRef(env, item_index);
+        free(mrl);
+    }
+
     /* Get the object class */
     jclass cls = (*env)->GetObjectClass(env, eventManagerInstance);
     if (!cls) {
@@ -192,9 +312,9 @@ static void vlc_event_callback(const libvlc_event_t *ev, void *data)
     }
 
     /* Find the callback ID */
-    jmethodID methodID = (*env)->GetMethodID(env, cls, "callback", "(II)V");
+    jmethodID methodID = (*env)->GetMethodID(env, cls, "callback", "(ILandroid/os/Bundle;)V");
     if (methodID) {
-        (*env)->CallVoidMethod(env, eventManagerInstance, methodID, ev->type, ev_opt_data);
+        (*env)->CallVoidMethod(env, eventManagerInstance, methodID, ev->type, bundle);
     } else {
         LOGE("EventManager: failed to get the callback method");
     }
@@ -289,6 +409,9 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
     jmethodID methodId = (*env)->GetMethodID(env, cls, "getAout", "()I");
     bool use_opensles = (*env)->CallIntMethod(env, thiz, methodId) == AOUT_OPENSLES;
 
+    methodId = (*env)->GetMethodID(env, cls, "timeStretchingEnabled", "()Z");
+    bool enable_time_stretch = (*env)->CallBooleanMethod(env, thiz, methodId);
+
     verbosity = verbose;
     libvlc_log_subscribe(&debug_subscriber, debug_log, &verbosity);
 
@@ -301,13 +424,12 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
         "--no-plugins-cache",
         "--no-drop-late-frames",
         "--avcodec-fast",
+        enable_time_stretch ? "--audio-time-stretch" : "--no-audio-time-stretch",
         use_opensles ? "--aout=opensles" : "--aout=android_audiotrack",
     };
     libvlc_instance_t *instance = libvlc_new(sizeof(argv) / sizeof(*argv), argv);
 
-    jclass clazz = (*env)->GetObjectClass(env, thiz);
-    jfieldID field = (*env)->GetFieldID(env, clazz, "mLibVlcInstance", "I");
-    (*env)->SetIntField(env, thiz, field, (jint) instance);
+    setLong(env, thiz, "mLibVlcInstance", (jlong) instance);
 
     if (!instance)
     {
@@ -316,6 +438,25 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
     }
 
     LOGI("LibVLC initialized: %p", instance);
+
+    /* Initialize media list (a.k.a. playlist/history) */
+    libvlc_media_list_t* pointer = libvlc_media_list_new( instance );
+    if(!pointer) {
+        jclass exc = (*env)->FindClass(env, "org/videolan/vlc/LibVlcException");
+        (*env)->ThrowNew(env, exc, "Unable to create LibVLC media list");
+        return;
+    }
+
+    /* Connect the event manager */
+    libvlc_event_manager_t *ev = libvlc_media_list_event_manager(pointer);
+    static const libvlc_event_type_t mp_events[] = {
+        libvlc_MediaListItemAdded,
+        libvlc_MediaListItemDeleted,
+    };
+    for(int i = 0; i < (sizeof(mp_events) / sizeof(*mp_events)); i++)
+        libvlc_event_attach(ev, mp_events[i], vlc_event_callback, myVm);
+
+    setLong(env, thiz, "mMediaListInstance", (jlong)pointer);
 }
 
 jstring Java_org_videolan_vlc_LibVLC_nativeToURI(JNIEnv *env, jobject thiz, jstring path)
@@ -324,7 +465,11 @@ jstring Java_org_videolan_vlc_LibVLC_nativeToURI(JNIEnv *env, jobject thiz, jstr
     /* Get C string */
     const char* psz_path = (*env)->GetStringUTFChars(env, path, &isCopy);
     /* Convert the path to URI */
-    char* psz_location = make_URI(psz_path, "file");
+    char* psz_location;
+    if(unlikely( strstr( psz_path, "://" ) ))
+        psz_location = strdup(psz_path);
+    else
+        psz_location = vlc_path2uri(psz_path, "file");
     /* Box into jstring */
     jstring t = (*env)->NewStringUTF(env, psz_location);
     /* Clean up */
@@ -336,9 +481,7 @@ jstring Java_org_videolan_vlc_LibVLC_nativeToURI(JNIEnv *env, jobject thiz, jstr
 void Java_org_videolan_vlc_LibVLC_nativeDestroy(JNIEnv *env, jobject thiz)
 {
     releaseMediaPlayer(env, thiz);
-    jclass clazz = (*env)->GetObjectClass(env, thiz);
-    jfieldID field = (*env)->GetFieldID(env, clazz, "mLibVlcInstance", "I");
-    jint libVlcInstance = (*env)->GetIntField(env, thiz, field);
+    jlong libVlcInstance = getLong(env, thiz, "mLibVlcInstance");
     if (!libVlcInstance)
         return; // Already destroyed
 
@@ -346,7 +489,7 @@ void Java_org_videolan_vlc_LibVLC_nativeDestroy(JNIEnv *env, jobject thiz)
     libvlc_release(instance);
     libvlc_log_unsubscribe(&debug_subscriber);
 
-    (*env)->SetIntField(env, thiz, field, 0);
+    setLong(env, thiz, "mLibVlcInstance", 0);
 }
 
 void Java_org_videolan_vlc_LibVLC_detachEventManager(JNIEnv *env, jobject thiz)
@@ -370,7 +513,7 @@ void Java_org_videolan_vlc_LibVLC_setEventManager(JNIEnv *env, jobject thiz, job
         return;
     }
 
-    jmethodID methodID = (*env)->GetMethodID(env, cls, "callback", "(II)V");
+    jmethodID methodID = (*env)->GetMethodID(env, cls, "callback", "(ILandroid/os/Bundle;)V");
     if (!methodID) {
         LOGE("setEventManager: failed to get the callback method");
         return;
@@ -379,77 +522,8 @@ void Java_org_videolan_vlc_LibVLC_setEventManager(JNIEnv *env, jobject thiz, job
     eventManagerInstance = (*env)->NewGlobalRef(env, eventManager);
 }
 
-void setInt(JNIEnv *env, jobject item, const char* field, int value)
-{
-    jclass cls;
-    jfieldID fieldId;
-
-    /* Get a reference to item's class */
-    cls = (*env)->GetObjectClass(env, item);
-
-    /* Look for the instance field s in cls */
-    fieldId = (*env)->GetFieldID(env, cls, field, "I");
-    if (fieldId == NULL)
-        return;
-
-    (*env)->SetIntField(env, item, fieldId, value);
-}
-
-void setLong(JNIEnv *env, jobject item, const char* field, long value)
-{
-    jclass cls;
-    jfieldID fieldId;
-
-    /* Get a reference to item's class */
-    cls = (*env)->GetObjectClass(env, item);
-
-    /* Look for the instance field s in cls */
-    fieldId = (*env)->GetFieldID(env, cls, field, "J");
-    if (fieldId == NULL)
-        return;
-
-    (*env)->SetLongField(env, item, fieldId, value);
-}
-
-void SetFloat(JNIEnv *env, jobject item, const char* field, float value)
-{
-    jclass cls;
-    jfieldID fieldId;
-
-    /* Get a reference to item's class */
-    cls = (*env)->GetObjectClass(env, item);
-
-    /* Look for the instance field s in cls */
-    fieldId = (*env)->GetFieldID(env, cls, field, "F");
-    if (fieldId == NULL)
-        return;
-
-    (*env)->SetFloatField(env, item, fieldId, value);
-}
-
-void setString(JNIEnv *env, jobject item, const char* field, const char* text)
-{
-    jclass cls;
-    jfieldID fieldId;
-    jstring jstr;
-
-    /* Get a reference to item's class */
-    cls = (*env)->GetObjectClass(env, item);
-
-    /* Look for the instance field s in cls */
-    fieldId = (*env)->GetFieldID(env, cls, field, "Ljava/lang/String;");
-    if (fieldId == NULL)
-        return;
-
-    /* Create a new string and overwrite the instance field */
-    jstr = (*env)->NewStringUTF(env, text);
-    if (jstr == NULL)
-        return;
-    (*env)->SetObjectField(env, item, fieldId, jstr);
-}
-
 jobjectArray Java_org_videolan_vlc_LibVLC_readMediaMeta(JNIEnv *env,
-                                                        jobject thiz, jint instance, jstring mrl)
+                                                        jobject thiz, jlong instance, jstring mrl)
 {
     jobjectArray array = (*env)->NewObjectArray(env, 8,
             (*env)->FindClass(env, "java/lang/String"),
@@ -490,27 +564,19 @@ jobjectArray Java_org_videolan_vlc_LibVLC_readMediaMeta(JNIEnv *env,
    return array;
 }
 
-void Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
-                                            jint instance, jstring mrl, jboolean novideo)
-{
+static void create_player_and_play(JNIEnv* env, jobject thiz,
+                                   jlong instance, int position) {
     /* Release previous media player, if any */
     releaseMediaPlayer(env, thiz);
 
-    /* Create a new item */
-    libvlc_media_t *m = new_media(instance, env, thiz, mrl, false, novideo);
-    if (!m)
-    {
-        LOGE("readMedia: Could not create the media!");
-        return;
-    }
+    libvlc_media_list_t* p_mlist = getMediaList(env, thiz);
 
     /* Create a media player playing environment */
+    libvlc_media_list_player_t* p_mlp = libvlc_media_list_player_new((libvlc_instance_t*)instance);
     libvlc_media_player_t *mp = libvlc_media_player_new((libvlc_instance_t*)instance);
 
     jobject myJavaLibVLC = (*env)->NewGlobalRef(env, thiz);
 
-    libvlc_media_player_set_media(mp, m);
-
     //if AOUT_AUDIOTRACK_JAVA, we use amem
     jclass cls = (*env)->GetObjectClass(env, thiz);
     jmethodID methodId = (*env)->GetMethodID(env, cls, "getAout", "()I");
@@ -521,9 +587,6 @@ void Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
         libvlc_audio_set_format_callbacks(mp, aout_open, aout_close);
     }
 
-    /* No need to keep the media now */
-    libvlc_media_release(m);
-
     /* Connect the event manager */
     libvlc_event_manager_t *ev = libvlc_media_player_event_manager(mp);
     static const libvlc_event_type_t mp_events[] = {
@@ -533,21 +596,89 @@ void Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
         libvlc_MediaPlayerStopped,
         libvlc_MediaPlayerVout,
     };
-    int i;
-    for (i = 0; i < (sizeof(mp_events) / sizeof(*mp_events)); ++i)
+    for(int i = 0; i < (sizeof(mp_events) / sizeof(*mp_events)); i++)
         libvlc_event_attach(ev, mp_events[i], vlc_event_callback, myVm);
 
-    /* Keep a pointer to this media player */
-    jclass clazz = (*env)->GetObjectClass(env, thiz);
-    jfieldID field = (*env)->GetFieldID(env, clazz,
-                                        "mMediaPlayerInstance", "I");
-    (*env)->SetIntField(env, thiz, field, (jint) mp);
+    libvlc_media_list_player_set_media_list(p_mlp, p_mlist);
+    libvlc_media_list_player_set_media_player(p_mlp, mp);
 
-    libvlc_media_player_play(mp);
+    /* Keep a pointer to this media player */
+    setLong(env, thiz, "mMediaListPlayerInstance", (jlong)p_mlp);
+    setLong(env, thiz, "mInternalMediaPlayerInstance", (jlong)mp);
+
+    libvlc_media_list_player_play_item_at_index(p_mlp, position);
+}
+
+jint Java_org_videolan_vlc_LibVLC_readMedia(JNIEnv *env, jobject thiz,
+                                            jlong instance, jstring mrl, jboolean novideo)
+{
+    /* Create a new item */
+    libvlc_media_t *m = new_media(instance, env, thiz, mrl, false, novideo);
+    if (!m)
+    {
+        LOGE("readMedia: Could not create the media!");
+        return;
+    }
+
+    libvlc_media_list_t* p_mlist = getMediaList(env, thiz);
+
+    libvlc_media_list_lock(p_mlist);
+    if(libvlc_media_list_add_media(p_mlist, m) != 0) {
+        LOGE("readMedia: Could not add to the media list!");
+        libvlc_media_list_unlock(p_mlist);
+        libvlc_media_release(m);
+        return;
+    }
+    int position = libvlc_media_list_index_of_item(p_mlist, m);
+    libvlc_media_list_unlock(p_mlist);
+
+    /* No need to keep the media now */
+    libvlc_media_release(m);
+
+    create_player_and_play(env, thiz, instance, position);
+
+    return position;
+}
+
+void Java_org_videolan_vlc_LibVLC_playIndex(JNIEnv *env, jobject thiz,
+                                            jlong instance, int position) {
+    create_player_and_play(env, thiz, instance, position);
+}
+
+void Java_org_videolan_vlc_LibVLC_getMediaListItems(
+                JNIEnv *env, jobject thiz, jobject arrayList) {
+    jclass arrayClass = (*env)->FindClass(env, "java/util/ArrayList");
+    jmethodID methodID = (*env)->GetMethodID(env, arrayClass, "add", "(Ljava/lang/Object;)Z");
+    jstring str;
+
+    libvlc_media_list_t* p_mlist = getMediaList(env, thiz);
+    libvlc_media_list_lock( p_mlist );
+    for(int i = 0; i < libvlc_media_list_count( p_mlist ); i++) {
+        char* mrl = libvlc_media_get_mrl( libvlc_media_list_item_at_index( p_mlist, i ) );
+        str = (*env)->NewStringUTF(env, mrl);
+        (*env)->CallBooleanMethod(env, arrayList, methodID, str);
+        (*env)->DeleteLocalRef(env, str);
+        free(mrl);
+    }
+    libvlc_media_list_unlock( p_mlist );
+}
+
+jfloat Java_org_videolan_vlc_LibVLC_getRate(JNIEnv *env, jobject thiz) {
+    libvlc_media_player_t* mp = getMediaPlayer(env, thiz);
+    if(mp)
+        return libvlc_media_player_get_rate(mp);
+    else
+        return 1.00;
+}
+
+void Java_org_videolan_vlc_LibVLC_setRate(JNIEnv *env, jobject thiz, jfloat rate) {
+    libvlc_media_player_t* mp = getMediaPlayer(env, thiz);
+    if(mp)
+        libvlc_media_player_set_rate(mp, rate);
 }
 
 jboolean Java_org_videolan_vlc_LibVLC_hasVideoTrack(JNIEnv *env, jobject thiz,
-                                                    jint i_instance, jstring fileLocation)
+                                                    jlong i_instance, jstring fileLocation)
 {
     /* Create a new item and assign it to the media player. */
     libvlc_media_t *p_m = new_media(i_instance, env, thiz, fileLocation, false, false);
@@ -612,8 +743,7 @@ jboolean Java_org_videolan_vlc_LibVLC_hasVideoTrack(JNIEnv *env, jobject thiz,
         return JNI_FALSE;
 }
 
-jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfo(JNIEnv *env, jobject thiz,
-                                                         jint instance, jstring mrl)
+jobjectArray read_track_info_internal(JNIEnv *env, jobject thiz, libvlc_media_t* p_m)
 {
     /* get java class */
     jclass cls = (*env)->FindClass( env, "org/videolan/vlc/TrackInfo" );
@@ -631,17 +761,8 @@ jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfo(JNIEnv *env, jobject th
         return NULL;
     }
 
-    /* Create a new item and assign it to the media player. */
-    libvlc_media_t *p_m = new_media(instance, env, thiz, mrl, false, false);
-    if (p_m == NULL)
-    {
-        LOGE("Could not create the media!");
-        return NULL;
-    }
-
     /* Get the tracks information of the media. */
     libvlc_media_track_info_t *p_tracks;
-    libvlc_media_parse(p_m);
 
     int i_nbTracks = libvlc_media_get_tracks_info(p_m, &p_tracks);
     jobjectArray array = (*env)->NewObjectArray(env, i_nbTracks + 1, cls, NULL);
@@ -678,7 +799,7 @@ jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfo(JNIEnv *env, jobject th
             {
                 setInt(env, item, "Height", p_tracks[i].u.video.i_height);
                 setInt(env, item, "Width", p_tracks[i].u.video.i_width);
-                SetFloat(env, item, "Framerate", p_tracks[i].u.video.f_frame_rate);
+                setFloat(env, item, "Framerate", p_tracks[i].u.video.f_frame_rate);
             }
             if (p_tracks[i].i_type == libvlc_track_audio)
             {
@@ -689,12 +810,42 @@ jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfo(JNIEnv *env, jobject th
     }
 
     libvlc_media_tracks_info_release(p_tracks, i_nbTracks);
-    libvlc_media_release(p_m);
     return array;
 }
 
+
+jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfo(JNIEnv *env, jobject thiz,
+                                                         jlong instance, jstring mrl)
+{
+    /* Create a new item and assign it to the media player. */
+    libvlc_media_t *p_m = new_media(instance, env, thiz, mrl, false, false);
+    if (p_m == NULL)
+    {
+        LOGE("Could not create the media!");
+        return NULL;
+    }
+
+    libvlc_media_parse(p_m);
+    jobjectArray jar = read_track_info_internal(env, thiz, p_m);
+    libvlc_media_release(p_m);
+    return jar;
+}
+
+
+jobjectArray Java_org_videolan_vlc_LibVLC_readTracksInfoPosition(JNIEnv *env, jobject thiz,
+                                                         jint position)
+{
+    libvlc_media_list_t* p_mlist = getMediaList(env, thiz);
+    libvlc_media_t *p_m = libvlc_media_list_item_at_index( p_mlist, position );
+    if (p_m == NULL) {
+        LOGE("Could not load get media @ position %d!", position);
+        return NULL;
+    } else
+        return read_track_info_internal(env, thiz, p_m);
+}
+
 jlong Java_org_videolan_vlc_LibVLC_getLengthFromLocation(JNIEnv *env, jobject thiz,
-                                                     jint i_instance, jstring fileLocation)
+                                                     jlong i_instance, jstring fileLocation)
 {
     jlong length = 0;
     struct length_change_monitor *monitor;
@@ -741,15 +892,16 @@ end:
 
 jboolean Java_org_videolan_vlc_LibVLC_hasMediaPlayer(JNIEnv *env, jobject thiz)
 {
-    return !!getMediaPlayer(env, thiz);
+    return !!getMediaListPlayer(env, thiz);
 }
 
 jboolean Java_org_videolan_vlc_LibVLC_isPlaying(JNIEnv *env, jobject thiz)
 {
-    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
     if (mp)
-        return !!libvlc_media_player_is_playing(mp);
-    return 0;
+        return !!libvlc_media_list_player_is_playing(mp);
+    else
+        return 0;
 }
 
 jboolean Java_org_videolan_vlc_LibVLC_isSeekable(JNIEnv *env, jobject thiz)
@@ -762,23 +914,37 @@ jboolean Java_org_videolan_vlc_LibVLC_isSeekable(JNIEnv *env, jobject thiz)
 
 void Java_org_videolan_vlc_LibVLC_play(JNIEnv *env, jobject thiz)
 {
-    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
     if (mp)
-        libvlc_media_player_play(mp);
+        libvlc_media_list_player_play(mp);
 }
 
 void Java_org_videolan_vlc_LibVLC_pause(JNIEnv *env, jobject thiz)
 {
-    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
     if (mp)
-        libvlc_media_player_pause(mp);
+        libvlc_media_list_player_pause(mp);
 }
 
 void Java_org_videolan_vlc_LibVLC_stop(JNIEnv *env, jobject thiz)
 {
-    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
     if (mp)
-        libvlc_media_player_stop(mp);
+        libvlc_media_list_player_stop(mp);
+}
+
+void Java_org_videolan_vlc_LibVLC_previous(JNIEnv *env, jobject thiz)
+{
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
+    if (mp)
+        libvlc_media_list_player_previous(mp);
+}
+
+void Java_org_videolan_vlc_LibVLC_next(JNIEnv *env, jobject thiz)
+{
+    libvlc_media_list_player_t *mp = getMediaListPlayer(env, thiz);
+    if (mp)
+        libvlc_media_list_player_next(mp);
 }
 
 jint Java_org_videolan_vlc_LibVLC_getVolume(JNIEnv *env, jobject thiz)
