@@ -44,8 +44,12 @@ public class LibVLC {
     private static int sAout = AOUT_AUDIOTRACK;
 
     /** libVLC instance C pointer */
-    private int mLibVlcInstance = 0; // Read-only, reserved for JNI
-    private int mMediaPlayerInstance = 0; // Read-only, reserved for JNI
+    private long mLibVlcInstance = 0; // Read-only, reserved for JNI
+    /** libvlc_media_list_player pointer */
+    private long mMediaListPlayerInstance = 0; // Read-only, reserved for JNI
+    private long mInternalMediaPlayerInstance = 0; // Read-only, reserved for JNI
+    /** libvlc_media_list_t pointer */
+    private long mMediaListInstance = 0; // Read-only, reserved for JNI
 
     private Aout mAout;
 
@@ -176,6 +180,11 @@ public class LibVLC {
         }
     }
 
+    public boolean timeStretchingEnabled() {
+        SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        return p.getBoolean("enable_time_stretching_audio", false);
+    }
+
     /**
      * Initialize the libVLC class
      */
@@ -189,7 +198,7 @@ public class LibVLC {
             Context context = VLCApplication.getAppContext();
             SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
             nativeInit(pref.getBoolean("enable_verbose_mode", true));
-            setEventManager(EventManager.getIntance());
+            setEventManager(EventManager.getInstance());
             mIsInitialized = true;
         }
     }
@@ -238,9 +247,18 @@ public class LibVLC {
     /**
      * Read a media.
      */
-    public void readMedia(String mrl, boolean novideo) {
+    public int readMedia(String mrl, boolean novideo) {
         Log.v(TAG, "Reading " + mrl);
-        readMedia(mLibVlcInstance, mrl, novideo);
+        return readMedia(mLibVlcInstance, mrl, novideo);
+    }
+
+    /**
+     * Play a media from the media list (playlist)
+     *
+     * @param position The index of the media
+     */
+    public void playIndex(int position) {
+        playIndex(mLibVlcInstance, position);
     }
 
     public String[] readMediaMeta(String mrl) {
@@ -272,6 +290,18 @@ public class LibVLC {
         return getLengthFromLocation(mLibVlcInstance, mrl);
     }
 
+    /**
+     * Sets the speed of playback (1 being normal speed, 2 being twice as fast)
+     *
+     * @param rate
+     */
+    public native void setRate(float rate);
+
+    /**
+     * Get the current playback speed
+     */
+    public native float getRate();
+
     /**
      * Change the verbosity of libvlc
      * @param verbose: true for increased verbosity
@@ -294,8 +324,14 @@ public class LibVLC {
      * Read a media
      * @param instance: the instance of libVLC
      * @param mrl: the media mrl
+     * @return the position in the playlist
      */
-    private native void readMedia(int instance, String mrl, boolean novideo);
+    private native int readMedia(long instance, String mrl, boolean novideo);
+
+    /**
+     * Play an index in the native media list (playlist)
+     */
+    private native void playIndex(long instance, int position);
 
     /**
      * Return true if there is currently a running media player.
@@ -327,6 +363,16 @@ public class LibVLC {
      */
     public native void stop();
 
+    /**
+     * Play the previous media (if any) in the media list
+     */
+    public native void previous();
+
+    /**
+     * Play the next media (if any) in the media list
+     */
+    public native void next();
+
     /**
      * Gets volume as integer
      */
@@ -391,16 +437,18 @@ public class LibVLC {
      * Get a media thumbnail.
      * @return a bytearray with the RGBA thumbnail data inside.
      */
-    private native byte[] getThumbnail(int instance, String mrl, int i_width, int i_height);
+    private native byte[] getThumbnail(long instance, String mrl, int i_width, int i_height);
 
     /**
      * Return true if there is a video track in the file
      */
-    private native boolean hasVideoTrack(int instance, String mrl);
+    private native boolean hasVideoTrack(long instance, String mrl);
 
-    private native String[] readMediaMeta(int instance, String mrl);
+    private native String[] readMediaMeta(long instance, String mrl);
 
-    private native TrackInfo[] readTracksInfo(int instance, String mrl);
+    private native TrackInfo[] readTracksInfo(long instance, String mrl);
+
+    public native TrackInfo[] readTracksInfoPosition(int position);
 
     public native int getAudioTracksCount();
 
@@ -426,10 +474,15 @@ public class LibVLC {
 
     public native boolean nativeIsPathDirectory(String path);
 
+    /**
+     * Get the list of existing items in the media list (playlist)
+     */
+    public native void getMediaListItems(ArrayList<String> arl);
+
     /**
      * Return the length of the stream, in milliseconds
      */
-    private native long getLengthFromLocation(int instance, String mrl);
+    private native long getLengthFromLocation(long instance, String mrl);
 
     private native void setEventManager(EventManager eventManager);
 
