@@ -22,6 +22,7 @@ package org.videolan.vlc;
 
 import java.io.File;
 import java.io.FileFilter;
+import java.io.IOException;
 import java.lang.Thread.State;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -81,7 +82,7 @@ public class MediaLibrary {
         ArrayList<Media> videoItems = new ArrayList<Media>();
         for (int i = 0; i < mItemList.size(); i++) {
             Media item = mItemList.get(i);
-            if (item.getType() == Media.TYPE_VIDEO) {
+            if (item != null && item.getType() == Media.TYPE_VIDEO) {
                 videoItems.add(item);
             }
         }
@@ -153,6 +154,7 @@ public class MediaLibrary {
     private class GetMediaItemsRunnable implements Runnable {
 
         private final Stack<File> directories = new Stack<File>();
+        private final HashSet<String> directoriesScanned = new HashSet<String>();
         private Context mContext;
 
         public GetMediaItemsRunnable(Context context) {
@@ -169,12 +171,15 @@ public class MediaLibrary {
             // use the external storage as our default root directory (most often /mnt/sdcard)
             if (root == null) {
                 root = Environment.getExternalStorageDirectory().getAbsolutePath();
+            } else {
+                root = new File(root).getAbsolutePath();
             }
 
-            // show progressbar in header
+            // show progressbar in footer
             MainActivity.showProgressBar(mContext);
 
             // get directories from database
+            mDBManager.removeDirNotUnder(root);
             directories.addAll(mDBManager.getMediaDirs());
             if (directories.isEmpty())
                 directories.add(new File(root));
@@ -191,65 +196,66 @@ public class MediaLibrary {
             MediaItemFilter mediaFileFilter = new MediaItemFilter();
 
             int count = 0;
-            int total = 0;
 
-            //first pass : count total files
+            ArrayList<File> mediaToScan = new ArrayList<File>();
+
+            // Count total files, and stack them
             while (!directories.isEmpty()) {
                 File dir = directories.pop();
+                String dirPath = dir.getAbsolutePath();
                 File[] f = null;
+
+                // Skip some system folders
+                if (dirPath.startsWith("/proc/") || dirPath.startsWith("/sys/") || dirPath.startsWith("/dev/"))
+                    continue;
+
+                // Do not scan again if same canonical path
+                try {
+                    dirPath = dir.getCanonicalPath();
+                } catch (IOException e) {
+                    e.printStackTrace();
+                }
+                if (directoriesScanned.contains(dirPath))
+                    continue;
+                else
+                    directoriesScanned.add(dirPath);
+
+                // Do no scan media in .nomedia folders
+                if (new File(dirPath + "/.nomedia").exists()) {
+                    continue;
+                }
+
+                // Filter the extensions and the folders
                 if ((f = dir.listFiles(mediaFileFilter)) != null) {
-                    for (int i = 0; i < f.length; i++) {
-                        File file = f[i];
+                    for (File file : f) {
                         if (file.isFile()) {
-                            total++;
+                            mediaToScan.add(file);
                         } else if (file.isDirectory()) {
                             directories.push(file);
                         }
                     }
                 }
             }
-            directories.addAll(mDBManager.getMediaDirs());
-            if (directories.isEmpty())
-                directories.add(new File(root));
 
-            //second pass : load Medias
-            while (!directories.isEmpty()) {
-                File dir = directories.pop();
-                File[] f = null;
-                /* .nomedia tells media players on Android to skip the
-                 * folder in the media library because they don't contain
-                 * useful music, such as notification sounds,
-                 * navigation voice phrases etc.
-                 */
-                if(new File(dir.getAbsolutePath() + "/.nomedia").exists()) {
-                    continue;
-                }
-
-                if ((f = dir.listFiles(mediaFileFilter)) != null) {
-                    for (int i = 0; i < f.length; i++) {
-                        File file = f[i];
-
-                        if (file.isFile()) {
-                            MainActivity.sendTextInfo(mContext, file.getName(), count, total);
-                            count++;
-                            String fileURI = Util.PathToURI(file.getPath());
-                            if (existingMedias.containsKey(fileURI)) {
-                                /** only add file if it is not already in the
-                                 * list. eg. if user select an subfolder as well
-                                 */
-                                if (!addedLocations.contains(fileURI)) {
-                                    // get existing media item from database
-                                    mItemList.add(existingMedias.get(fileURI));
-                                    addedLocations.add(fileURI);
-                                }
-                            } else {
-                                // create new media item
-                                mItemList.add(new Media(fileURI, true));
-                            }
-                        } else if (file.isDirectory()) {
-                            directories.push(file);
-                        }
+            // Process the stacked items
+            for (File file : mediaToScan) {
+                String fileURI = Util.PathToURI(file.getPath());
+                MainActivity.sendTextInfo(mContext, file.getName(), count,
+                        mediaToScan.size());
+                count++;
+                if (existingMedias.containsKey(fileURI)) {
+                    /**
+                     * only add file if it is not already in the list. eg. if
+                     * user select an subfolder as well
+                     */
+                    if (!addedLocations.contains(fileURI)) {
+                        // get existing media item from database
+                        mItemList.add(existingMedias.get(fileURI));
+                        addedLocations.add(fileURI);
                     }
+                } else {
+                    // create new media item
+                    mItemList.add(new Media(fileURI, true));
                 }
             }
 
@@ -259,15 +265,19 @@ public class MediaLibrary {
                 h.sendEmptyMessage(MEDIA_ITEMS_UPDATED);
             }
 
-            // remove file from database if storage is mounted
+            // remove old files & folders from database if storage is mounted
             if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
                 for (String fileURI : addedLocations) {
                     existingMedias.remove(fileURI);
                 }
                 mDBManager.removeMedias(existingMedias.keySet());
+
+                for (File file : mDBManager.getMediaDirs())
+                    if (!file.isDirectory())
+                        mDBManager.removeDir(file.getAbsolutePath());
             }
 
-            // hide progressbar in header
+            // hide progressbar in footer
             MainActivity.clearTextInfo(mContext);
             MainActivity.hideProgressBar(mContext);
             mContext = null;
