@@ -282,10 +282,6 @@ public class DirectoryAdapter extends BaseAdapter {
         else
             holder.icon.setImageResource(R.drawable.ic_folder);
 
-        if(selectedNode.isFile())
-            v.setLongClickable(true);
-        else
-            v.setLongClickable(false);
         return v;
     }
 
@@ -315,6 +311,8 @@ public class DirectoryAdapter extends BaseAdapter {
     }
 
     public String getMediaLocation(int position) {
+        if (position >= mCurrentNode.children.size())
+            return null;
         return LibVLC.getExistingInstance().nativeToURI(
                 this.mCurrentDir + "/" + mCurrentNode.children.get(position).name
         );
