@@ -27,12 +27,12 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.LibVLC;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
+import org.videolan.vlc.VlcRunnable;
 import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.interfaces.ISortable;
 
 import android.app.AlertDialog;
-import android.content.Intent;
 import android.os.Bundle;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
@@ -84,15 +84,25 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
 
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
-        MenuInflater menuInflater = getActivity().getMenuInflater();
-        menuInflater.inflate(R.menu.directory_view, menu);
+        int position = ((AdapterContextMenuInfo)menuInfo).position;
+
+        if(mDirectoryAdapter.isChildFile(position)) {
+            MenuInflater menuInflater = getActivity().getMenuInflater();
+            menuInflater.inflate(R.menu.directory_view, menu);
+        }
     }
 
     @Override
     public boolean onContextItemSelected(MenuItem item) {
         AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
+        if(info == null) // info can be null
+            return super.onContextItemSelected(item);
+
         int id = item.getItemId();
         String mediaLocation = mDirectoryAdapter.getMediaLocation(info.position);
+        if (mediaLocation == null)
+            return super.onContextItemSelected(item);
+
         if(id == R.id.directory_view_play) {
             openMediaFile(info.position);
             return true;
@@ -103,20 +113,22 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
             audioController.append(tmp);
             return true;
         } else if(id == R.id.directory_view_delete) {
-            AlertDialog alertDialog = CommonDialogs.deleteMedia(getActivity(), mediaLocation, null);
+            AlertDialog alertDialog = CommonDialogs.deleteMedia(getActivity(), mediaLocation,
+                    new VlcRunnable() {
+                        @Override
+                        public void run(Object o) {
+                            refresh();
+                        }
+                    });
             alertDialog.show();
         } else if(id == R.id.directory_view_play_audio) {
             AudioServiceController audioController = AudioServiceController.getInstance();
             ArrayList<String> arrayList = new ArrayList<String>();
             arrayList.add(mediaLocation);
             audioController.load(arrayList, 0);
-            Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-            startActivity(intent);
+            AudioPlayerActivity.start(getActivity());
         } else if(id == R.id.directory_view_play_video) {
-            Intent intent = new Intent(getActivity(), VideoPlayerActivity.class);
-            intent.putExtra("itemLocation", mediaLocation);
-            startActivity(intent);
+            VideoPlayerActivity.start(getActivity(), mediaLocation);
         }
         return super.onContextItemSelected(item);
     }
@@ -136,14 +148,9 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
         try {
             if(!LibVLC.getExistingInstance().hasVideoTrack(mediaFile)) {
                 audioController.load(mDirectoryAdapter.getAllMediaLocations(), p-1); /* p-1 to exclude ".," */
-                Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-                startActivity(intent);
+                AudioPlayerActivity.start(getActivity());
             } else {
-                audioController.stop();
-                Intent intent = new Intent(getActivity(), VideoPlayerActivity.class);
-                intent.putExtra("itemLocation", mediaFile);
-                startActivity(intent);
+                VideoPlayerActivity.start(getActivity(), mediaFile);
             }
         } catch (IOException e) {
             /* disk error maybe? */
