@@ -30,15 +30,13 @@ import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCCallbackTask;
-import org.videolan.vlc.gui.audio.AudioBrowserFragment;
+import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.video.VideoListAdapter;
-import org.videolan.vlc.gui.video.VideoListFragment;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.widget.AudioMiniPlayer;
 
 import android.app.AlertDialog;
 import android.app.Dialog;
-import android.app.ProgressDialog;
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.DialogInterface;
@@ -48,26 +46,35 @@ import android.content.SharedPreferences;
 import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageInfo;
 import android.content.pm.PackageManager.NameNotFoundException;
+import android.content.res.Configuration;
 import android.os.Bundle;
+import android.os.Handler;
 import android.preference.PreferenceManager;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentTransaction;
 import android.util.Log;
+import android.view.Display;
+import android.view.LayoutInflater;
 import android.view.View;
 import android.view.View.OnClickListener;
+import android.view.WindowManager;
+import android.widget.AdapterView;
+import android.widget.AdapterView.OnItemClickListener;
 import android.widget.Button;
 import android.widget.CheckBox;
 import android.widget.EditText;
+import android.widget.ImageView;
+import android.widget.ListView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
 import com.actionbarsherlock.app.ActionBar;
-import com.actionbarsherlock.app.ActionBar.Tab;
 import com.actionbarsherlock.app.SherlockFragmentActivity;
 import com.actionbarsherlock.view.Menu;
 import com.actionbarsherlock.view.MenuInflater;
 import com.actionbarsherlock.view.MenuItem;
 import com.actionbarsherlock.view.Window;
+import com.slidingmenu.lib.SlidingMenu;
 
 public class MainActivity extends SherlockFragmentActivity {
     public final static String TAG = "VLC/MainActivity";
@@ -76,23 +83,26 @@ public class MainActivity extends SherlockFragmentActivity {
     protected static final String ACTION_HIDE_PROGRESSBAR = "org.videolan.vlc.gui.HideProgressBar";
     protected static final String ACTION_SHOW_TEXTINFO = "org.videolan.vlc.gui.ShowTextInfo";
 
-    private static final int VIDEO_TAB = 0;
-    private static final int AUDIO_TAB = 1;
     private static final String PREF_SHOW_INFO = "show_info";
+    private static final String PREF_FIRST_RUN = "first_run";
 
     private ActionBar mActionBar;
+    private SlidingMenu mMenu;
+    private SidebarAdapter mSidebarAdapter;
     private AudioMiniPlayer mAudioPlayer;
     private AudioServiceController mAudioController;
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
     private TextView mInfoText;
-    private DirectoryViewFragment mDirectoryView;
-    private int mCurrentViewTab;
-    private Boolean mMediaLibraryActive;
+    private String mCurrentFragment;
 
     private SharedPreferences mSettings;
 
     private int mVersionNumber = -1;
+    private boolean mFirstRun = false;
+
+    public MainActivity() {
+    }
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
@@ -107,45 +117,105 @@ public class MainActivity extends SherlockFragmentActivity {
 
         if (Util.isICSOrLater()) /* Bug on pre-ICS, the progress bar is always present */
             requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
-        setContentView(R.layout.main);
 
-        super.onCreate(savedInstanceState);
+        // Set up the sliding menu
+        setContentView(R.layout.sliding_menu);
+        mMenu = (SlidingMenu) findViewById(R.id.sliding_menu);
+        updateMenuOffset();
+
+        View v_main = LayoutInflater.from(this).inflate(R.layout.main, null);
+        mMenu.setViewAbove(v_main);
+        View sidebar = LayoutInflater.from(this).inflate(R.layout.sidebar, null);
+        ((ListView)sidebar).setFooterDividersEnabled(true);
+        final ListView listView = (ListView)sidebar.findViewById(android.R.id.list);
+        mSidebarAdapter = new SidebarAdapter(getSupportFragmentManager());
+        listView.setAdapter(mSidebarAdapter);
+        mMenu.setViewBehind(sidebar);
+
+        /* Get the current version from package */
+        PackageInfo pinfo = null;
+        try {
+            pinfo = getPackageManager().getPackageInfo(getPackageName(), 0);
+        } catch (NameNotFoundException e) {
+            Log.e(TAG, "package info not found.");
+        }
+        if (pinfo != null)
+            mVersionNumber = pinfo.versionCode;
 
         /* Get settings */
         mSettings = PreferenceManager.getDefaultSharedPreferences(this);
+
+        /* Check if it's the first run */
+        mFirstRun = mSettings.getInt(PREF_FIRST_RUN, -1) != mVersionNumber;
+        if (mFirstRun) {
+            Editor editor = mSettings.edit();
+            editor.putInt(PREF_FIRST_RUN, mVersionNumber);
+            editor.commit();
+        }
+
         LibVLC.useIOMX(this);
+        try {
+            // Start LibVLC
+            LibVLC.getInstance();
+        } catch (LibVlcException e) {
+            e.printStackTrace();
+            super.onCreate(null);
+            Intent i = new Intent(this, CompatErrorActivity.class);
+            i.putExtra("runtimeError", true);
+            i.putExtra("message", "LibVLC failed to initialize (LibVlcException)");
+            startActivity(i);
+            finish();
+            return;
+        }
+
+        super.onCreate(savedInstanceState);
 
         /* Initialize variables */
-        mInfoLayout = findViewById(R.id.info_layout);
-        mInfoProgress = (ProgressBar) findViewById(R.id.info_progress);
-        mInfoText = (TextView) findViewById(R.id.info_text);
+        mInfoLayout = v_main.findViewById(R.id.info_layout);
+        mInfoProgress = (ProgressBar) v_main.findViewById(R.id.info_progress);
+        mInfoText = (TextView) v_main.findViewById(R.id.info_text);
 
-        /* Initialize the tabs */
+        /* Set up the action bar */
         mActionBar = getSupportActionBar();
-        mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
+        mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
         mActionBar.setDisplayOptions(0, ActionBar.DISPLAY_SHOW_TITLE);
+        mActionBar.setDisplayHomeAsUpEnabled(true);
 
-        mActionBar.addTab(mActionBar.newTab()
-                .setText("Video")
-                .setIcon(R.drawable.header_icon_video)
-                .setTabListener(new TabListener<VideoListFragment>(
-                        this, "video", VideoListFragment.class)));
+        /* Add padding between the home button and the arrow */
+        ImageView home = (ImageView)findViewById(Util.isHoneycombOrLater()
+                ? android.R.id.home : R.id.abs__home);
+        if (home != null) 
+            home.setPadding(20, 0, 0, 0);
 
-        mActionBar.addTab(mActionBar.newTab()
-                .setText("Audio")
-                .setIcon(R.drawable.header_icon_audio)
-                .setTabListener(new TabListener<AudioBrowserFragment>(
-                        this, "audio", AudioBrowserFragment.class)));
+        /* Set up the sidebar click listener */
+        listView.setOnItemClickListener(new OnItemClickListener() {
 
-        /* DirectoryView */
-        mDirectoryView = new DirectoryViewFragment();
-        mDirectoryView.setRetainInstance(true); /* Retain instance across attach/detach */
-        getSupportFragmentManager().beginTransaction()
-            .add(R.id.fragment_placeholder, mDirectoryView)
-            .commit();
-        getSupportFragmentManager().beginTransaction()
-            .detach(mDirectoryView)
-            .commit();
+            @Override
+            public void onItemClick (AdapterView<?> parent, View view,
+                    int position, long id) {
+                SidebarAdapter.SidebarEntry entry = (SidebarEntry) listView.getItemAtPosition(position);
+                Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
+                if(current.getTag() == entry.id) /* Already selected */
+                    return;
+
+                /* Clear any backstack before switching tabs.
+                 * This way it's more consistent for the user, who might have
+                 * switched tabs and hit back to quit, only to activate an old
+                 * backstack.
+                 */
+                if(getSupportFragmentManager().getBackStackEntryCount() > 0) {
+                    for(int i = 0; i < getSupportFragmentManager().getBackStackEntryCount(); i++) {
+                        getSupportFragmentManager().popBackStack();
+                    }
+                }
+                FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
+                ft.detach(current);
+                ft.attach(mSidebarAdapter.getFragment(entry.id));
+                ft.commit();
+                mCurrentFragment = entry.id;
+                mMenu.showAbove();
+            }
+        });
 
         /* Set up the mini audio player */
         mAudioPlayer = new AudioMiniPlayer();
@@ -158,17 +228,21 @@ public class MainActivity extends SherlockFragmentActivity {
             .commit();
 
         /* Show info/alpha/beta Warning */
-        PackageInfo pinfo = null;
-        try {
-            pinfo = getPackageManager().getPackageInfo(getPackageName(), 0);
-        } catch (NameNotFoundException e) {
-            Log.e(TAG, "package info not found.");
-        }
-        if (pinfo != null) {
-            mVersionNumber = pinfo.versionCode;
-
-            if (mSettings.getInt(PREF_SHOW_INFO, -1) != mVersionNumber)
-                showInfoDialog();
+        if (mSettings.getInt(PREF_SHOW_INFO, -1) != mVersionNumber)
+            showInfoDialog();
+        else if (mFirstRun) {
+            /*
+             * The sliding menu is automatically opened when the user closes
+             * the info dialog. If (for any reason) the dialog is not shown,
+             * open the menu after a short delay.
+             */
+            final Handler handler = new Handler();
+            handler.postDelayed(new Runnable() {
+                @Override
+                public void run() {
+                    mMenu.showBehind();
+                }
+            }, 500);
         }
 
         /* Prepare the progressBar */
@@ -185,6 +259,14 @@ public class MainActivity extends SherlockFragmentActivity {
         MediaLibrary.getInstance(this).loadMediaItems(this);
     }
 
+    private void updateMenuOffset() {
+        WindowManager wm = (WindowManager) getSystemService(WINDOW_SERVICE);
+        Display display = wm.getDefaultDisplay();
+        @SuppressWarnings("deprecation")
+        int behindOffset_dp = Util.convertPxToDp(display.getWidth()) - 208;
+        mMenu.setBehindOffset(Util.convertDpToPx(behindOffset_dp));
+    }
+
     @Override
     protected void onResume() {
         mAudioController.addAudioPlayer(mAudioPlayer);
@@ -192,12 +274,29 @@ public class MainActivity extends SherlockFragmentActivity {
         Boolean startFromNotification = getIntent().hasExtra(AudioService.START_FROM_NOTIFICATION);
 
         /* Restore last view */
-        if(!mMediaLibraryActive)
-            showDirectoryView();
-        else if (startFromNotification || mCurrentViewTab == AUDIO_TAB)
-            showAudioTab();
-        else
-            showVideoTab();
+        Fragment current = getSupportFragmentManager()
+                .findFragmentById(R.id.fragment_placeholder);
+        boolean found = false;
+        if(current != null) {
+            for(int i = 0; i < SidebarAdapter.entries.size(); i++) {
+                if(SidebarAdapter.entries.get(i).id == current.getTag()) {
+                    found = true;
+                    break;
+                }
+            }
+        } else {
+            found = true;
+        }
+        /* Don't call replace() on a non-sidebar fragment, since replace() will
+         * remove() the currently displayed fragment and replace it with a
+         * blank screen.
+         */
+        if(found) {
+            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
+            ft.replace(R.id.fragment_placeholder, mSidebarAdapter.getFragment(mCurrentFragment));
+            ft.commit();
+        }
+
         if (startFromNotification)
             getIntent().removeExtra(AudioService.START_FROM_NOTIFICATION);
         super.onResume();
@@ -209,8 +308,7 @@ public class MainActivity extends SherlockFragmentActivity {
     @Override
     protected void onPause() {
         SharedPreferences.Editor editor = getSharedPreferences("MainActivity", MODE_PRIVATE).edit();
-        editor.putInt("tab", mCurrentViewTab);
-        editor.putBoolean("medialibrary", mMediaLibraryActive);
+        editor.putString("fragment", mCurrentFragment);
         editor.commit();
         mAudioController.removeAudioPlayer(mAudioPlayer);
         super.onPause();
@@ -231,6 +329,16 @@ public class MainActivity extends SherlockFragmentActivity {
         reloadPreferences();
     }
 
+    @Override
+    public void onBackPressed() {
+        if(mMenu.isBehindShowing()) {
+            /* Close the menu first */
+            mMenu.showAbove();
+        } else {
+            super.onBackPressed();
+        }
+    }
+
     /** Create menu from XML
      */
     @Override
@@ -241,11 +349,15 @@ public class MainActivity extends SherlockFragmentActivity {
          */
         MenuInflater inflater = getSupportMenuInflater();
         inflater.inflate(R.menu.media_library, menu);
-        MenuItem browse = menu.findItem(R.id.ml_menu_browse);
-        browse.setTitle(mMediaLibraryActive ? R.string.directories : R.string.media_library);
         return super.onCreateOptionsMenu(menu);
     }
 
+    @Override
+    public void onConfigurationChanged(Configuration newConfig) {
+        super.onConfigurationChanged(newConfig);
+        updateMenuOffset();
+    }
+
     @Override
     public boolean onSearchRequested() {
         Intent intent = new Intent(this, SearchActivity.class);
@@ -286,24 +398,15 @@ public class MainActivity extends SherlockFragmentActivity {
                 break;
             // Refresh
             case R.id.ml_menu_refresh:
-                if(mMediaLibraryActive)
-                    MediaLibrary.getInstance(this).loadMediaItems(this);
+                // TODO: factor this into each fragment
+                if(mCurrentFragment.equals("directories")) {
+                    DirectoryViewFragment directoryView = (DirectoryViewFragment) mSidebarAdapter.getFragment(mCurrentFragment);
+                    directoryView.refresh();
+                }
+                else if(mCurrentFragment.equals("history"))
+                    ((HistoryFragment)mSidebarAdapter.getFragment(mCurrentFragment)).refresh();
                 else
-                    mDirectoryView.refresh();
-                break;
-            // Browse Folders
-            case R.id.ml_menu_browse:
-                if(mMediaLibraryActive) {
-                    item.setTitle(R.string.media_library);
-                    showDirectoryView();
-                }
-                else {
-                    item.setTitle(R.string.directories);
-                    if (mCurrentViewTab == AUDIO_TAB)
-                        showAudioTab();
-                    else
-                        showVideoTab();
-                }
+                    MediaLibrary.getInstance(this).loadMediaItems(this);
                 break;
             // Open MRL
             case R.id.ml_menu_open_mrl:
@@ -312,42 +415,20 @@ public class MainActivity extends SherlockFragmentActivity {
             case R.id.ml_menu_search:
             	onSearchRequested();
             	break;
+            case android.R.id.home:
+                /* Toggle the sidebar */
+                if(mMenu.isBehindShowing())
+                    mMenu.showAbove();
+                else
+                    mMenu.showBehind();
+                break;
         }
         return super.onOptionsItemSelected(item);
     }
 
     private void reloadPreferences() {
         SharedPreferences sharedPrefs = getSharedPreferences("MainActivity", MODE_PRIVATE);
-        mCurrentViewTab = sharedPrefs.getInt("tab", VIDEO_TAB);
-        mMediaLibraryActive = sharedPrefs.getBoolean("medialibrary", true);
-    }
-
-    private void showDirectoryView() {
-        FragmentTransaction ft = this.getSupportFragmentManager().beginTransaction();
-
-        ft.setCustomAnimations(R.anim.anim_enter_bottom, 0);
-
-        /* Remove existing tabs */
-        if (mActionBar.getNavigationMode() != ActionBar.NAVIGATION_MODE_STANDARD)
-            mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
-        /* Load directory view fragment */
-        ft.attach(mDirectoryView);
-        ft.commitAllowingStateLoss();
-        mMediaLibraryActive = false;
-    }
-
-    private void hideDirectoryView() {
-        FragmentTransaction ft = this.getSupportFragmentManager().beginTransaction();
-
-        ft.setCustomAnimations(0, R.anim.anim_leave_bottom);
-
-        /* Restore the tabs */
-        if (mActionBar.getNavigationMode() != ActionBar.NAVIGATION_MODE_TABS)
-            mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
-        /* Remove the directory view from the tabs */
-        ft.detach(mDirectoryView);
-        ft.commitAllowingStateLoss();
-        mMediaLibraryActive = true;
+        mCurrentFragment = sharedPrefs.getString("fragment", "video");
     }
 
     private void showInfoDialog() {
@@ -364,35 +445,16 @@ public class MainActivity extends SherlockFragmentActivity {
                     editor.putInt(PREF_SHOW_INFO, mVersionNumber);
                     editor.commit();
                 }
+                /* Close the dialog */
                 infoDialog.dismiss();
+                /* and finally open the sliding menu if first run */
+                if (mFirstRun)
+                    mMenu.showBehind();
             }
         });
         infoDialog.show();
     }
 
-    /**
-     * onClick event from xml
-     * @param view
-     */
-    public void changeTabClick(View view) {
-        // Toggle audio- and video-tab
-        if (mActionBar.getSelectedNavigationIndex() == VIDEO_TAB) {
-            showAudioTab();
-        } else {
-            showVideoTab();
-        }
-    }
-
-    private void showVideoTab() {
-        hideDirectoryView();
-        mActionBar.setSelectedNavigationItem(VIDEO_TAB);
-    }
-
-    private void showAudioTab() {
-        hideDirectoryView();
-        mActionBar.setSelectedNavigationItem(AUDIO_TAB);
-    }
-
     /**
      * onClick event from xml
      * @param view
@@ -463,40 +525,27 @@ public class MainActivity extends SherlockFragmentActivity {
         b.setPositiveButton(R.string.open, new DialogInterface.OnClickListener() {
             @Override
             public void onClick(DialogInterface dialog, int button) {
-                ProgressDialog pd = ProgressDialog.show(
-                        MainActivity.this,
-                        getApplicationContext().getString(R.string.loading),
-                        "Please wait...", true);
-                pd.setCancelable(true);
 
-                VLCCallbackTask t = new VLCCallbackTask(
-                    /* Task to run */
-                    new VLCCallbackTask.CallbackListener() {
-                        @Override
-                        public void callback() {
-                            AudioServiceController c = AudioServiceController.getInstance();
-                            String s = input.getText().toString();
+                /* Start this in a new thread as to not block the UI thread */
+                VLCCallbackTask task = new VLCCallbackTask(MainActivity.this)
+                {
+                    @Override
+                    public void run() {
+                      AudioServiceController c = AudioServiceController.getInstance();
+                      String s = input.getText().toString();
 
-                            /* Use the audio player by default. If a video track is
-                             * detected, then it will automatically switch to the video
-                             * player. This allows us to support more types of streams
-                             * (for example, RTSP and TS streaming) where ES can be
-                             * dynamically adapted rather than a simple scan.
-                             */
-                            ArrayList<String> media = new ArrayList<String>();
-                            media.add(s);
-                            c.append(media);
-                        }
-
-                        @Override
-                        public void callback_object(Object o) {
-                            ProgressDialog pd = (ProgressDialog)o;
-                            pd.dismiss();
-                        }
-                    }, pd);
-
-                /* Start this in a new friend as to not block the UI thread */
-                new Thread(t).start();
+                      /* Use the audio player by default. If a video track is
+                       * detected, then it will automatically switch to the video
+                       * player. This allows us to support more types of streams
+                       * (for example, RTSP and TS streaming) where ES can be
+                       * dynamically adapted rather than a simple scan.
+                       */
+                      ArrayList<String> media = new ArrayList<String>();
+                      media.add(s);
+                      c.append(media);
+                    }
+                };
+                task.execute();
             }
         }
         );
@@ -507,74 +556,4 @@ public class MainActivity extends SherlockFragmentActivity {
                 }});
         b.show();
     }
-
-    public static class TabListener<T extends Fragment> implements ActionBar.TabListener {
-        private final SherlockFragmentActivity mActivity;
-        private final String mTag;
-        private final Class<T> mClass;
-        private final Bundle mArgs;
-        private Fragment mFragment;
-
-        public TabListener(SherlockFragmentActivity activity, String tag, Class<T> clz) {
-            this(activity, tag, clz, null);
-        }
-
-        public TabListener(SherlockFragmentActivity activity, String tag, Class<T> clz, Bundle args) {
-            mActivity = activity;
-            mTag = tag;
-            mClass = clz;
-            mArgs = args;
-
-            // Check to see if we already have a fragment for this tab, probably
-            // from a previously saved state.  If so, deactivate it, because our
-            // initial state is that a tab isn't shown.
-            mFragment = mActivity.getSupportFragmentManager().findFragmentByTag(mTag);
-            if (mFragment != null && !mFragment.isDetached()) {
-                FragmentTransaction ft = mActivity.getSupportFragmentManager().beginTransaction();
-                ft.detach(mFragment);
-                ft.commit();
-            }
-        }
-
-        @Override
-        public void onTabSelected(Tab tab, FragmentTransaction ft) {
-            Fragment current = mActivity.getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
-
-            if (mTag.equalsIgnoreCase("video"))
-                ((MainActivity)mActivity).mCurrentViewTab = VIDEO_TAB;
-            else if (mTag.equalsIgnoreCase("audio"))
-                ((MainActivity)mActivity).mCurrentViewTab = AUDIO_TAB;
-
-            if (current != null && current.getTag() != null) {
-                if (current.getTag() == mTag)
-                    return;
-                if (mTag.equalsIgnoreCase("video"))
-                    ft.setCustomAnimations(0, R.anim.anim_leave_right);
-                else if (mTag.equalsIgnoreCase("audio"))
-                    ft.setCustomAnimations(0, R.anim.anim_leave_left);
-                ft.detach(current);
-            }
-
-            if (mTag.equalsIgnoreCase("video"))
-                ft.setCustomAnimations(R.anim.anim_enter_left, 0);
-            else if (mTag.equalsIgnoreCase("audio"))
-                ft.setCustomAnimations(R.anim.anim_enter_right, 0);
-
-            if (mFragment == null) {
-                mFragment = Fragment.instantiate(mActivity, mClass.getName(), mArgs);
-                mFragment.setRetainInstance(true);
-                ft.add(R.id.fragment_placeholder, mFragment, mTag);
-            } else {
-                ft.attach(mFragment);
-            }
-        }
-
-        @Override
-        public void onTabUnselected(Tab tab, FragmentTransaction ft) {
-        }
-
-        @Override
-        public void onTabReselected(Tab tab, FragmentTransaction ft) {
-        }
-    }
 }
