@@ -28,7 +28,8 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
-import org.videolan.vlc.VLCCallbackTask;
+import org.videolan.vlc.Util;
+import org.videolan.vlc.VlcRunnable;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.CommonDialogs;
 import org.videolan.vlc.interfaces.ISortable;
@@ -36,7 +37,6 @@ import org.videolan.vlc.widget.FlingViewGroup;
 import org.videolan.vlc.widget.FlingViewGroup.ViewSwitchListener;
 
 import android.app.AlertDialog;
-import android.content.Intent;
 import android.graphics.Color;
 import android.os.Bundle;
 import android.os.Handler;
@@ -170,9 +170,7 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         @Override
         public void onItemClick(AdapterView<?> av, View v, int p, long id) {
             mAudioController.load(mSongsAdapter.getLocations(), p);
-            Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-            startActivity(intent);
+            AudioPlayerActivity.start(getActivity());
         }
     };
 
@@ -237,12 +235,19 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         MenuInflater inflater = getActivity().getMenuInflater();
         inflater.inflate(R.menu.audio_list_browser, menu);
 
-        if (v.getId() != R.id.songs_list)
+        if (v.getId() != R.id.songs_list) {
             menu.setGroupEnabled(R.id.songs_view_only, false);
+            menu.setGroupEnabled(R.id.phone_only, false);
+        }
+        if (!Util.isPhone())
+            menu.setGroupVisible(R.id.phone_only, false);
     }
 
     @Override
     public boolean onContextItemSelected(MenuItem item) {
+        ContextMenuInfo menuInfo = item.getMenuInfo();
+        if(menuInfo == null) return super.onContextItemSelected(item);
+
         int startPosition;
         int groupPosition;
         int childPosition;
@@ -254,7 +259,6 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         boolean append = (id == R.id.audio_list_browser_append ||
                           id == R.id.audio_list_browser_append_all);
 
-        ContextMenuInfo menuInfo = item.getMenuInfo();
         if (ExpandableListContextMenuInfo.class.isInstance(menuInfo)) {
             ExpandableListContextMenuInfo info = (ExpandableListContextMenuInfo) menuInfo;
             groupPosition = ExpandableListView.getPackedPositionGroup(info.packedPosition);
@@ -269,23 +273,26 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         }
 
         if (id == R.id.audio_list_browser_delete) {
-            AlertDialog alertDialog = CommonDialogs.deleteMedia(getActivity(), mSongsAdapter.getLocation(groupPosition).get(0),
-                    new VLCCallbackTask(new VLCCallbackTask.CallbackListener() {
-
-                @Override
-                public void callback_object(Object o) {
-                    Media aMedia = (Media)o;
-                    mMediaLibrary.getMediaItems().remove(aMedia);
-                    updateLists();
-                }
-
-                @Override
-                public void callback() { }
-            }, mSongsAdapter.getItem(groupPosition)));
+            AlertDialog alertDialog = CommonDialogs.deleteMedia(
+                    getActivity(),
+                    mSongsAdapter.getLocation(groupPosition).get(0),
+                    new VlcRunnable(mSongsAdapter.getItem(groupPosition)) {
+                        @Override
+                        public void run(Object o) {
+                            Media aMedia = (Media) o;
+                            mMediaLibrary.getMediaItems().remove(aMedia);
+                            updateLists();
+                        }
+                    });
             alertDialog.show();
             return true;
         }
 
+        if (id == R.id.audio_list_browser_set_song) {
+            AudioUtil.setRingtone(mSongsAdapter.getItem(groupPosition),getActivity());
+            return true;
+        }
+
         if (useAllItems) {
             startPosition = groupPosition;
             medias = mSongsAdapter.getLocations();
@@ -316,9 +323,7 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         else
             mAudioController.load(medias, startPosition);
 
-        Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-        startActivity(intent);
+        AudioPlayerActivity.start(getActivity());
         return super.onContextItemSelected(item);
     }
 
@@ -372,6 +377,8 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         @Override
         public void handleMessage(Message msg) {
             AudioBrowserFragment fragment = getOwner();
+            if(fragment == null) return;
+
             switch (msg.what) {
                 case MediaLibrary.MEDIA_ITEMS_UPDATED:
                     fragment.updateLists();
