@@ -20,16 +20,17 @@
 
 package org.videolan.vlc.gui.video;
 
+import java.util.HashMap;
 import java.util.List;
 import java.util.concurrent.BrokenBarrierException;
 import java.util.concurrent.CyclicBarrier;
 
-import org.videolan.vlc.AudioServiceController;
+import org.videolan.vlc.DatabaseManager;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
 import org.videolan.vlc.ThumbnailerManager;
-import org.videolan.vlc.VLCCallbackTask;
+import org.videolan.vlc.VlcRunnable;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.CommonDialogs;
 import org.videolan.vlc.gui.PreferencesActivity;
@@ -105,8 +106,8 @@ public class VideoListFragment extends SherlockListFragment implements ISortable
         //Get & highlight the last media
         SharedPreferences preferences = getActivity().getSharedPreferences(PreferencesActivity.NAME, Context.MODE_PRIVATE);
         String lastPath = preferences.getString(PreferencesActivity.LAST_MEDIA, null);
-        long lastTime = preferences.getLong(PreferencesActivity.LAST_TIME, 0);
-        mVideoAdapter.setLastMedia(lastTime, lastPath);
+        HashMap<String, Long> times = DatabaseManager.getInstance(getActivity()).getVideoTimes(getActivity());
+        mVideoAdapter.setLastMedia(lastPath, times);
         mVideoAdapter.notifyDataSetChanged();
         mMediaLibrary.addUpdateHandler(mHandler);
         super.onResume();
@@ -128,20 +129,12 @@ public class VideoListFragment extends SherlockListFragment implements ISortable
     }
 
     protected void playVideo(int position) {
-        // Stop the currently running audio
-        AudioServiceController asc = AudioServiceController.getInstance();
-        asc.stop();
-
         Media item = (Media) getListAdapter().getItem(position);
-        Intent intent = new Intent(getActivity(), VideoPlayerActivity.class);
-        intent.putExtra("itemLocation", item.getLocation());
-        startActivity(intent);
+        VideoPlayerActivity.start(getActivity(), item.getLocation());
     }
 
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
-        super.onCreateContextMenu(menu, v, menuInfo);
-
         MenuInflater inflater = getActivity().getMenuInflater();
         inflater.inflate(R.menu.video_list, menu);
     }
@@ -165,15 +158,12 @@ public class VideoListFragment extends SherlockListFragment implements ISortable
                 AlertDialog alertDialog = CommonDialogs.deleteMedia(
                         getActivity(),
                         mVideoAdapter.getItem(positionDelete).getLocation(),
-                        new VLCCallbackTask(new VLCCallbackTask.CallbackListener() {
-                    @Override
-                    public void callback() {
-                        mVideoAdapter.remove(mVideoAdapter.getItem(positionDelete));
-                    }
-
-                    @Override
-                    public void callback_object(Object o) { }
-                }));
+                        new VlcRunnable() {
+                            @Override
+                            public void run(Object o) {
+                                mVideoAdapter.remove(mVideoAdapter.getItem(positionDelete));
+                            }
+                        });
                 alertDialog.show();
                 return true;
         }
@@ -200,6 +190,8 @@ public class VideoListFragment extends SherlockListFragment implements ISortable
         @Override
         public void handleMessage(Message msg) {
             VideoListFragment fragment = getOwner();
+            if(fragment == null) return;
+
             switch (msg.what) {
                 case UPDATE_ITEM:
                     fragment.updateItem();
