@@ -26,23 +26,26 @@ import java.lang.reflect.Method;
 import java.net.URLDecoder;
 
 import org.videolan.vlc.AudioServiceController;
+import org.videolan.vlc.DatabaseManager;
 import org.videolan.vlc.EventManager;
 import org.videolan.vlc.LibVLC;
 import org.videolan.vlc.LibVlcException;
+import org.videolan.vlc.Media;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.PreferencesActivity;
+import org.videolan.vlc.gui.SpeedSelectorDialog;
 import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.interfaces.IPlayerControl;
 import org.videolan.vlc.interfaces.OnPlayerControlListener;
 import org.videolan.vlc.widget.PlayerControlClassic;
 import org.videolan.vlc.widget.PlayerControlWheel;
+import org.videolan.vlc.widget.SlidingPanel;
 
 import android.annotation.TargetApi;
 import android.app.Activity;
-import android.app.AlertDialog;
-import android.app.AlertDialog.Builder;
+import android.app.KeyguardManager;
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.DialogInterface;
@@ -51,6 +54,7 @@ import android.content.IntentFilter;
 import android.content.SharedPreferences;
 import android.content.pm.ActivityInfo;
 import android.content.res.Configuration;
+import android.graphics.Color;
 import android.graphics.PixelFormat;
 import android.media.AudioManager;
 import android.os.Build;
@@ -58,7 +62,9 @@ import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.preference.PreferenceManager;
+import android.provider.Settings.SettingNotFoundException;
 import android.text.format.DateFormat;
+import android.util.DisplayMetrics;
 import android.util.Log;
 import android.view.Display;
 import android.view.MotionEvent;
@@ -69,13 +75,21 @@ import android.view.SurfaceView;
 import android.view.View;
 import android.view.View.OnClickListener;
 import android.view.View.OnSystemUiVisibilityChangeListener;
+import android.view.View.OnTouchListener;
 import android.view.ViewGroup.LayoutParams;
 import android.view.WindowManager;
 import android.view.animation.AnimationUtils;
+import android.widget.AdapterView;
+import android.widget.AdapterView.OnItemSelectedListener;
+import android.widget.ArrayAdapter;
 import android.widget.FrameLayout;
 import android.widget.ImageButton;
 import android.widget.SeekBar;
 import android.widget.SeekBar.OnSeekBarChangeListener;
+import android.widget.SlidingDrawer.OnDrawerCloseListener;
+import android.widget.SlidingDrawer.OnDrawerOpenListener;
+import android.widget.SlidingDrawer.OnDrawerScrollListener;
+import android.widget.Spinner;
 import android.widget.TextView;
 
 public class VideoPlayerActivity extends Activity {
@@ -99,7 +113,9 @@ public class VideoPlayerActivity extends Activity {
     /** Overlay */
     private View mOverlayHeader;
     private View mOverlay;
+    private SlidingPanel mOverlaySlider;
     private static final int OVERLAY_TIMEOUT = 4000;
+    private static final int OVERLAY_INFINITE = 3600000;
     private static final int FADE_OUT = 1;
     private static final int SHOW_PROGRESS = 2;
     private static final int SURFACE_SIZE = 3;
@@ -116,10 +132,13 @@ public class VideoPlayerActivity extends Activity {
     private TextView mLength;
     private TextView mInfo;
     private IPlayerControl mControls;
-    private ImageButton mAudio;
-    private ImageButton mSubtitles;
+    private boolean mEnableWheelbar;
+    private boolean mEnableBrightnessGesture;
+    private Spinner mAudio;
+    private Spinner mSubtitles;
     private ImageButton mLock;
     private ImageButton mSize;
+    private TextView mSpeedLabel;
 
     /**
      * For uninterrupted switching between audio and video mode
@@ -127,19 +146,29 @@ public class VideoPlayerActivity extends Activity {
     private boolean mSwitchingView;
     private boolean mEndReached;
 
+    // Playlist
+    private int savedIndexPosition = -1;
+
     // size of the video
     private int mVideoHeight;
     private int mVideoWidth;
+    private int mSarNum;
+    private int mSarDen;
 
     //Audio
     private AudioManager mAudioManager;
     private int mAudioMax;
     private int mAudioDisplayRange;
-    private float mTouchY, mVol;
-    private boolean mIsAudioChanged;
+    private float mTouchY, mTouchX, mVol;
     private String[] mAudioTracks;
     private String[] mSubtitleTracks;
 
+    //Audio Or Brightness
+    private boolean mIsAudioOrBrightnessChanged;
+
+    // Brightness
+    private boolean mIsFirstBrightnessGesture = true;
+
     @Override
     @TargetApi(11)
     protected void onCreate(Bundle savedInstanceState) {
@@ -150,10 +179,11 @@ public class VideoPlayerActivity extends Activity {
         if(Util.isICSOrLater())
             getWindow().getDecorView().findViewById(android.R.id.content).setOnSystemUiVisibilityChangeListener(
                     new OnSystemUiVisibilityChangeListener() {
+                        @Override
                         public void onSystemUiVisibilityChange(int visibility) {
                             if (visibility == mUiVisibility)
                                 return;
-                            setSurfaceSize(mVideoWidth, mVideoHeight);
+                            setSurfaceSize(mVideoWidth, mVideoHeight, mSarNum, mSarDen);
                             if (visibility == View.SYSTEM_UI_FLAG_VISIBLE && !mShowing) {
                                 showOverlay();
                                 mHandler.sendMessageDelayed(mHandler.obtainMessage(HIDE_NAV), OVERLAY_TIMEOUT);
@@ -166,6 +196,49 @@ public class VideoPlayerActivity extends Activity {
         /** initialize Views an their Events */
         mOverlayHeader = findViewById(R.id.player_overlay_header);
         mOverlay = findViewById(R.id.player_overlay);
+        mOverlaySlider = (SlidingPanel) findViewById(R.id.player_overlay_slider);
+
+        /* prevent touch event from going through the slider view */
+        View sliderContent = findViewById(R.id.slider_content);
+        sliderContent.setOnTouchListener(new OnTouchListener() {
+            @Override
+            public boolean onTouch(View v, MotionEvent event) {
+                return true;
+            }
+        });
+
+        mOverlaySlider.setOnDrawerScrollListener(new OnDrawerScrollListener() {
+            @Override
+            public void onScrollStarted() {
+                showOverlay(OVERLAY_INFINITE);
+            }
+
+            @Override
+            public void onScrollEnded() {
+                if (mOverlaySlider.isOpened()) {
+                    mOverlaySlider.ExpandHandle();
+                    showOverlay(OVERLAY_INFINITE);
+                }
+                else {
+                    mOverlaySlider.CollapseHandle();
+                    showOverlay(OVERLAY_TIMEOUT);
+                }
+            }
+        });
+        mOverlaySlider.setOnDrawerOpenListener(new OnDrawerOpenListener() {
+            @Override
+            public void onDrawerOpened() {
+                mOverlaySlider.ExpandHandle();
+                showOverlay(OVERLAY_INFINITE);
+            }
+        });
+        mOverlaySlider.setOnDrawerCloseListener(new OnDrawerCloseListener() {
+            @Override
+            public void onDrawerClosed() {
+                mOverlaySlider.CollapseHandle();
+                showOverlay(OVERLAY_TIMEOUT);
+            }
+        });
 
         /* header */
         mTitle = (TextView) findViewById(R.id.player_overlay_title);
@@ -177,18 +250,17 @@ public class VideoPlayerActivity extends Activity {
         // the info textView is not on the overlay
         mInfo = (TextView) findViewById(R.id.player_overlay_info);
 
-        mControls = pref.getBoolean("enable_wheel_bar", false)
+        mEnableWheelbar = pref.getBoolean("enable_wheel_bar", false);
+        mEnableBrightnessGesture = pref.getBoolean("enable_gesture_brightness", true);
+        mControls = mEnableWheelbar
                 ? new PlayerControlWheel(this)
                 : new PlayerControlClassic(this);
         mControls.setOnPlayerControlListener(mPlayerControlListener);
         FrameLayout mControlContainer = (FrameLayout) findViewById(R.id.player_control);
         mControlContainer.addView((View) mControls);
 
-        mAudio = (ImageButton) findViewById(R.id.player_overlay_audio);
-        mAudio.setOnClickListener(mAudioListener);
-
-        mSubtitles = (ImageButton) findViewById(R.id.player_overlay_subtitle);
-        mSubtitles.setOnClickListener(mSubtitlesListener);
+        mAudio = (Spinner) findViewById(R.id.player_overlay_audio);
+        mSubtitles = (Spinner) findViewById(R.id.player_overlay_subtitle);
 
         mLock = (ImageButton) findViewById(R.id.player_overlay_lock);
         mLock.setOnClickListener(mLockListener);
@@ -196,6 +268,9 @@ public class VideoPlayerActivity extends Activity {
         mSize = (ImageButton) findViewById(R.id.player_overlay_size);
         mSize.setOnClickListener(mSizeListener);
 
+        mSpeedLabel = (TextView) findViewById(R.id.player_overlay_speed);
+        mSpeedLabel.setOnClickListener(mSpeedLabelListener);
+
         mSurface = (SurfaceView) findViewById(R.id.player_surface);
         mSurfaceHolder = mSurface.getHolder();
         mSurfaceHolder.setFormat(PixelFormat.RGBX_8888);
@@ -219,13 +294,12 @@ public class VideoPlayerActivity extends Activity {
             e.printStackTrace();
         }
 
-        EventManager em = EventManager.getIntance();
+        EventManager em = EventManager.getInstance();
         em.addHandler(eventHandler);
 
         this.setVolumeControlStream(AudioManager.STREAM_MUSIC);
 
         setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
-        load();
     }
 
     @Override
@@ -241,11 +315,31 @@ public class VideoPlayerActivity extends Activity {
             super.onPause();
             return;
         }
-        long time = 0;
-        if (mLibVLC.isPlaying()) {
-            time = mLibVLC.getTime() - 5000;
+        long time = mLibVLC.getTime();
+        long length = mLibVLC.getLength();
+        //remove saved position if in the last 5 seconds
+        if (length - time < 5000)
+            time = 0;
+        else
+            time -= 5000; // go back 5 seconds, to compensate loading time
+
+        /*
+         * Pausing here generates errors because the vout is constantly
+         * trying to refresh itself every 80ms while the surface is not
+         * accessible anymore.
+         * To workaround that, we keep the last known position in the playlist
+         * in savedIndexPosition to be able to restore it during onResume().
+         */
+        if (savedIndexPosition >= 0)
+            mLibVLC.stop();
+        else {
+            /* FIXME when the playback is started externally from AudioService
+             * we don't have a savedIndexPosition. Use pause as a fallback until
+             * we find a solution.
+             */
             mLibVLC.pause();
         }
+
         mSurface.setKeepScreenOn(false);
 
         // Save position
@@ -253,8 +347,11 @@ public class VideoPlayerActivity extends Activity {
             SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
             SharedPreferences.Editor editor = preferences.edit();
             editor.putString(PreferencesActivity.LAST_MEDIA, mLocation);
-            editor.putLong(PreferencesActivity.LAST_TIME, time);
             editor.commit();
+            DatabaseManager.getInstance(this).updateMedia(
+                    mLocation,
+                    DatabaseManager.mediaColumn.MEDIA_TIME,
+                    time);
         }
         super.onPause();
     }
@@ -266,7 +363,7 @@ public class VideoPlayerActivity extends Activity {
             mLibVLC.stop();
         }
 
-        EventManager em = EventManager.getIntance();
+        EventManager em = EventManager.getInstance();
         em.removeHandler(eventHandler);
 
         mAudioManager = null;
@@ -274,9 +371,7 @@ public class VideoPlayerActivity extends Activity {
         if(mSwitchingView) {
             Log.d(TAG, "mLocation = \"" + mLocation + "\"");
             AudioServiceController.getInstance().showWithoutParse(mLocation);
-            Intent i = new Intent(this, AudioPlayerActivity.class);
-            i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
-            startActivity(i);
+            AudioPlayerActivity.start(this, true);
         }
         //AudioServiceController.getInstance().unbindAudioService(this);
         super.onDestroy();
@@ -285,15 +380,62 @@ public class VideoPlayerActivity extends Activity {
     @Override
     protected void onResume() {
         AudioServiceController.getInstance().bindAudioService(this);
+
+        load();
+
+        /*
+         * if the activity has been paused by pressing the power button,
+         * pressing it again will show the lock screen.
+         * But onResume will also be called, even if vlc-android is still in the background.
+         * To workaround that, pause playback if the lockscreen is displayed
+         */
+        mHandler.postDelayed(new Runnable() {
+            public void run() {
+                if (mLibVLC != null && mLibVLC.isPlaying()) {
+                    KeyguardManager km = (KeyguardManager)getSystemService(KEYGUARD_SERVICE);
+                    if (km.inKeyguardRestrictedInputMode())
+                        mLibVLC.pause();
+                }
+            }}, 500);
+
+        showOverlay();
+
         super.onResume();
     }
 
+    public static void start(Context context, String location) {
+        start(context, location, null, false);
+    }
+
+    public static void start(Context context, String location, String title, Boolean dontParse) {
+        Intent intent = new Intent(context, VideoPlayerActivity.class);
+        intent.putExtra("itemLocation", location);
+        intent.putExtra("itemTitle", title);
+        intent.putExtra("dontParse", dontParse);
+
+        if (dontParse)
+            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
+        else {
+            // Stop the currently running audio
+            AudioServiceController asc = AudioServiceController.getInstance();
+            asc.stop();
+        }
+
+        context.startActivity(intent);
+    }
+
     private final BroadcastReceiver mBatteryReceiver = new BroadcastReceiver()
     {
         @Override
         public void onReceive(Context context, Intent intent)
         {
             int batteryLevel = intent.getIntExtra("level", 0);
+            if (batteryLevel >= 50)
+                mBattery.setTextColor(Color.GREEN);
+            else if  (batteryLevel >= 30)
+                mBattery.setTextColor(Color.YELLOW);
+            else
+                mBattery.setTextColor(Color.RED);
             mBattery.setText(String.format("%d%%", batteryLevel));
         }
     };
@@ -306,14 +448,16 @@ public class VideoPlayerActivity extends Activity {
 
     @Override
     public void onConfigurationChanged(Configuration newConfig) {
-        setSurfaceSize(mVideoWidth, mVideoHeight);
+        setSurfaceSize(mVideoWidth, mVideoHeight, mSarNum, mSarDen);
         super.onConfigurationChanged(newConfig);
     }
 
-    public void setSurfaceSize(int width, int height) {
+    public void setSurfaceSize(int width, int height, int sar_num, int sar_den) {
         // store video size
         mVideoHeight = height;
         mVideoWidth = width;
+        mSarNum = sar_num;
+        mSarDen = sar_den;
         Message msg = mHandler.obtainMessage(SURFACE_SIZE);
         mHandler.sendMessage(msg);
     }
@@ -324,7 +468,7 @@ public class VideoPlayerActivity extends Activity {
     private void lockScreen() {
         WindowManager wm = (WindowManager) getSystemService(WINDOW_SERVICE);
         Display display = wm.getDefaultDisplay();
-        int rotation = display.getOrientation();
+        int rotation;
         if (Build.VERSION.SDK_INT >= 8 /* Android 2.2 has getRotation */) {
             try {
                 Method m = display.getClass().getDeclaredMethod("getRotation");
@@ -425,6 +569,8 @@ public class VideoPlayerActivity extends Activity {
         @Override
         public void handleMessage(Message msg) {
             VideoPlayerActivity activity = getOwner();
+            if(activity == null) return;
+
             switch (msg.getData().getInt("event")) {
                 case EventManager.MediaPlayerPlaying:
                     Log.i(TAG, "MediaPlayerPlaying");
@@ -463,6 +609,9 @@ public class VideoPlayerActivity extends Activity {
         @Override
         public void handleMessage(Message msg) {
             VideoPlayerActivity activity = getOwner();
+            if(activity == null) // WeakReference could be GC'ed early
+                return;
+
             switch (msg.what) {
                 case FADE_OUT:
                     activity.hideOverlay(false);
@@ -521,9 +670,20 @@ public class VideoPlayerActivity extends Activity {
         if (dw * dh == 0)
             return;
 
-        // calculate aspect ratio
-        double ar = (double) mVideoWidth / (double) mVideoHeight;
-        // calculate display aspect ratio
+        // compute the aspect ratio
+        double ar, vw;
+        double density = (double)mSarNum / (double)mSarDen;
+        if (density == 1.0) {
+            /* No indication about the density, assuming 1:1 */
+            vw = mVideoWidth;
+            ar = (double)mVideoWidth / (double)mVideoHeight;
+        } else {
+            /* Use the specified aspect ratio */
+            vw = (double)mVideoWidth * density;
+            ar = vw / (double)mVideoHeight;
+        }
+
+        // compute the display aspect ratio
         double dar = (double) dw / (double) dh;
 
         switch (mCurrentSize) {
@@ -557,7 +717,7 @@ public class VideoPlayerActivity extends Activity {
                 break;
             case SURFACE_ORIGINAL:
                 dh = mVideoHeight;
-                dw = mVideoWidth;
+                dw = (int) vw;
                 break;
         }
 
@@ -575,44 +735,134 @@ public class VideoPlayerActivity extends Activity {
 
     @Override
     public boolean onTouchEvent(MotionEvent event) {
+        DisplayMetrics screen = new DisplayMetrics();
+        getWindowManager().getDefaultDisplay().getMetrics(screen);
 
         if (mAudioDisplayRange == 0)
-            mAudioDisplayRange = Math.min(
-                    getWindowManager().getDefaultDisplay().getWidth(),
-                    getWindowManager().getDefaultDisplay().getHeight());
+            mAudioDisplayRange = Math.min(screen.widthPixels, screen.heightPixels);
+
+        float y_changed = event.getRawY() - mTouchY;
+        float x_changed = event.getRawX() - mTouchX;
+
+        // coef is the gradient's move to determine a neutral zone
+        float coef = Math.abs (y_changed / x_changed);
+        float xgesturesize = ((x_changed / screen.xdpi) * 2.54f);
+        float ygesturesize = ((y_changed / screen.ydpi) * 2.54f);
 
         switch (event.getAction()) {
 
-            case MotionEvent.ACTION_DOWN:
-                mTouchY = event.getRawY();
-                mVol = mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
-                mIsAudioChanged = false;
-                break;
+        case MotionEvent.ACTION_DOWN:
+            // Audio
+            mTouchY = event.getRawY();
+            mVol = mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
+            mIsAudioOrBrightnessChanged = false;
+            // Seek
+            mTouchX = event.getRawX();
+            break;
 
-            case MotionEvent.ACTION_MOVE:
-                float y = event.getRawY();
-
-                int delta = (int) (((mTouchY - y) / mAudioDisplayRange) * mAudioMax);
-                int vol = (int) Math.min(Math.max(mVol + delta, 0), mAudioMax);
-                if (delta != 0) {
-                    mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC, vol, AudioManager.FLAG_SHOW_UI);
-                    mIsAudioChanged = true;
-                }
-                break;
-
-            case MotionEvent.ACTION_UP:
-                if (!mIsAudioChanged) {
-                    if (!mShowing) {
-                        showOverlay();
-                    } else {
-                        hideOverlay(true);
-                        if(Util.isICSOrLater())
-                            mHandler.sendMessageDelayed(mHandler.obtainMessage(HIDE_NAV), OVERLAY_TIMEOUT);
+        case MotionEvent.ACTION_MOVE:
+            // No audio/brightness action if coef < 2
+            if (coef > 2) {
+                // Audio (Up or Down - Right side)
+                if (!mEnableBrightnessGesture || mTouchX > (screen.widthPixels / 2)){
+                    int delta = -(int) ((y_changed / mAudioDisplayRange) * mAudioMax);
+                    int vol = (int) Math.min(Math.max(mVol + delta, 0), mAudioMax);
+                    if (delta != 0) {
+                        mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC,
+                                vol, AudioManager.FLAG_SHOW_UI);
+                        mIsAudioOrBrightnessChanged = true;
                     }
                 }
-                break;
+                // Brightness (Up or Down - Left side)
+                if (mEnableBrightnessGesture && mTouchX < (screen.widthPixels / 2)){
+                    if (mIsFirstBrightnessGesture) initBrightnessTouch();
+                    doBrightnessTouch( - ygesturesize);
+                }
+            }
+            // Seek (Right or Left move)
+            evalTouchSeek(coef, xgesturesize, false);
+            break;
+
+        case MotionEvent.ACTION_UP:
+            // Audio or Brightness
+            if (!mIsAudioOrBrightnessChanged) {
+                if (!mShowing) {
+                    showOverlay();
+                } else {
+                    hideOverlay(true);
+                    if (Util.isICSOrLater())
+                        mHandler.sendMessageDelayed(
+                                mHandler.obtainMessage(HIDE_NAV),
+                                OVERLAY_TIMEOUT);
+                }
+            }
+            // Seek
+            evalTouchSeek(coef, xgesturesize, true);
+            break;
         }
-        return mIsAudioChanged;
+        return mIsAudioOrBrightnessChanged;
+    }
+
+    private void evalTouchSeek(float coef, float gesturesize, boolean seek) {
+        // No seek action if coef > 0.5 and gesturesize < 1cm
+        if (mEnableWheelbar || coef > 0.5 || Math.abs(gesturesize) < 1)
+            return;
+
+        // Always show seekbar when searching
+        if (!mShowing) showOverlay();
+
+        long length = mLibVLC.getLength();
+        long time = mLibVLC.getTime();
+
+        // Size of the jump, 10 minutes max (600000), with a bi-cubic progression, for a 8cm gesture
+        int jump = (int) (Math.signum(gesturesize) * ((600000 * Math.pow((gesturesize / 8), 4)) + 3000));
+
+        // Adjust the jump
+        if ((jump > 0) && ((time + jump) > length))
+            jump = (int) (length - time);
+        if ((jump < 0) && ((time + jump) < 0))
+            jump = (int) -time;
+
+        //Jump !
+        if (seek)
+            mPlayerControlListener.onSeekTo(time + jump);
+
+        //Show the jump's size
+        showInfo(String.format("%s%s (%s)",
+                jump >= 0 ? "+" : "",
+                Util.millisToString(jump),
+                Util.millisToString(time + jump)), 1000);
+    }
+
+    private void initBrightnessTouch() {
+        float brightnesstemp = 0.01f;
+        // Initialize the layoutParams screen brightness
+        try {
+            brightnesstemp = android.provider.Settings.System.getInt(getContentResolver(),
+                    android.provider.Settings.System.SCREEN_BRIGHTNESS) / 255.0f;
+        } catch (SettingNotFoundException e) {
+            // TODO Auto-generated catch block
+            e.printStackTrace();
+        }
+        WindowManager.LayoutParams lp = getWindow().getAttributes();
+        lp.screenBrightness = brightnesstemp;
+        getWindow().setAttributes(lp);
+        mIsFirstBrightnessGesture = false;
+    }
+    
+    private void doBrightnessTouch(float gesturesize) {
+        // No Brightness action if gesturesize < 0.4 cm
+        if (Math.abs(gesturesize) < 0.4)
+            return;
+
+        mIsAudioOrBrightnessChanged = true;
+        WindowManager.LayoutParams lp = getWindow().getAttributes();
+        lp.screenBrightness += Math.signum(gesturesize) * 0.05f;
+        // Adjust Brightness
+        if (lp.screenBrightness > 1) lp.screenBrightness = 1;
+        else if (lp.screenBrightness <= 0) lp.screenBrightness = 0.01f;
+        // Set Brightness
+        getWindow().setAttributes(lp);
     }
 
     /**
@@ -623,7 +873,7 @@ public class VideoPlayerActivity extends Activity {
         @Override
         public void onStartTrackingTouch(SeekBar seekBar) {
             mDragging = true;
-            showOverlay(3600000);
+            showOverlay(OVERLAY_INFINITE);
         }
 
         @Override
@@ -648,50 +898,30 @@ public class VideoPlayerActivity extends Activity {
     /**
      *
      */
-    private final OnClickListener mAudioListener = new OnClickListener() {
+    private final OnItemSelectedListener mAudioListener = new OnItemSelectedListener() {
         @Override
-        public void onClick(View v) {
-            if (mAudioTracks == null || mAudioTracks.length <= 1)
-                return;
+        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
+            mLibVLC.setAudioTrack(position + 1);
+        }
 
-            int current = mLibVLC.getAudioTrack() - 1;
-
-            Builder builder = new AlertDialog.Builder(VideoPlayerActivity.this);
-            builder.setSingleChoiceItems(mAudioTracks, current, new DialogInterface.OnClickListener() {
-                @Override
-                public void onClick(DialogInterface dialog, int which) {
-                    dialog.dismiss();
-                    mLibVLC.setAudioTrack(which + 1);
-                }
-            });
-
-            builder.show();
+        @Override
+        public void onNothingSelected(AdapterView<?> parent) {
         }
     };
 
     /**
     *
     */
-   private final OnClickListener mSubtitlesListener = new OnClickListener() {
-       @Override
-    public void onClick(View v) {
-           if (mSubtitleTracks == null || mSubtitleTracks.length == 0)
-               return;
+    private final OnItemSelectedListener mSubtitlesListener = new OnItemSelectedListener() {
+        @Override
+        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
+            mLibVLC.setSpuTrack(position);
+        }
 
-           int current = mLibVLC.getSpuTrack();
-
-           Builder builder = new AlertDialog.Builder(VideoPlayerActivity.this);
-           builder.setSingleChoiceItems(mSubtitleTracks, current, new DialogInterface.OnClickListener() {
-               @Override
-               public void onClick(DialogInterface dialog, int which) {
-                   dialog.dismiss();
-                   mLibVLC.setSpuTrack(which);
-               }
-           });
-
-           builder.show();
-       }
-   };
+        @Override
+        public void onNothingSelected(AdapterView<?> parent) {
+        }
+    };
 
 
     /**
@@ -709,8 +939,9 @@ public class VideoPlayerActivity extends Activity {
 
         @Override
         public void onSeek(int delta) {
-            long position = mLibVLC.getTime();
-            mLibVLC.setTime(position + delta);
+            long position = mLibVLC.getTime() + delta;
+            if (position < 0) position = 0;
+            mLibVLC.setTime(position);
             showOverlay();
         }
 
@@ -722,7 +953,7 @@ public class VideoPlayerActivity extends Activity {
 
         @Override
         public long onWheelStart() {
-            showOverlay(3600000);
+            showOverlay(OVERLAY_INFINITE);
             return mLibVLC.getTime();
         }
 
@@ -796,6 +1027,21 @@ public class VideoPlayerActivity extends Activity {
         }
     };
 
+    private final OnClickListener mSpeedLabelListener = new OnClickListener() {
+        @Override
+        public void onClick(View v) {
+            SpeedSelectorDialog d = new SpeedSelectorDialog(VideoPlayerActivity.this);
+            d.setOnDismissListener(new DialogInterface.OnDismissListener() {
+
+                @Override
+                public void onDismiss(DialogInterface dialog) {
+                    mSpeedLabel.setText(String.format(java.util.Locale.US, "%.2fx", LibVLC.getExistingInstance().getRate()));
+                }
+            });
+            d.show();
+        }
+    };
+
     /**
      * attach and disattach surface to the lib
      */
@@ -832,6 +1078,7 @@ public class VideoPlayerActivity extends Activity {
             mShowing = true;
             mOverlayHeader.setVisibility(View.VISIBLE);
             mOverlay.setVisibility(View.VISIBLE);
+            mOverlaySlider.setVisibility(View.VISIBLE);
             dimStatusBar(false);
         }
         Message msg = mHandler.obtainMessage(FADE_OUT);
@@ -841,17 +1088,35 @@ public class VideoPlayerActivity extends Activity {
         }
         if (mAudioTracks == null) {
             mAudioTracks = mLibVLC.getAudioTrackDescription();
-            if (mAudioTracks != null && mAudioTracks.length > 1)
-                mAudio.setVisibility(View.VISIBLE);
-            else
-                mAudio.setVisibility(View.GONE);
+            if (mAudioTracks != null && mAudioTracks.length > 1) {
+                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAudioTracks);
+                mAudio.setAdapter(adapter);
+                int current = mLibVLC.getAudioTrack() - 1;
+                mAudio.setSelection(current);
+                mAudio.setOnItemSelectedListener(mAudioListener);
+                mAudio.setEnabled(true);
+            }
+            else {
+                mAudio.setEnabled(false);
+                mAudio.setOnItemSelectedListener(null);
+                mAudio.setAdapter(null);
+            }
         }
         if (mSubtitleTracks == null) {
             mSubtitleTracks = mLibVLC.getSpuTrackDescription();
-            if (mSubtitleTracks != null && mSubtitleTracks.length > 0)
-                mSubtitles.setVisibility(View.VISIBLE);
-            else
-                mSubtitles.setVisibility(View.GONE);
+            if (mSubtitleTracks != null && mSubtitleTracks.length > 0) {
+                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mSubtitleTracks);
+                mSubtitles.setAdapter(adapter);
+                int current = mLibVLC.getSpuTrack();
+                mSubtitles.setSelection(current);
+                mSubtitles.setOnItemSelectedListener(mSubtitlesListener);
+                mSubtitles.setEnabled(true);
+            }
+            else {
+                mSubtitles.setEnabled(false);
+                mSubtitles.setOnItemSelectedListener(null);
+                mSubtitles.setAdapter(null);
+            }
         }
         updateOverlayPausePlay();
     }
@@ -866,9 +1131,11 @@ public class VideoPlayerActivity extends Activity {
             if (!fromUser) {
                 mOverlayHeader.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlay.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
+                mOverlaySlider.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
             }
             mOverlayHeader.setVisibility(View.INVISIBLE);
             mOverlay.setVisibility(View.INVISIBLE);
+            mOverlaySlider.setVisibility(View.INVISIBLE);
             mShowing = false;
             dimStatusBar(true);
         }
@@ -882,7 +1149,7 @@ public class VideoPlayerActivity extends Activity {
     private void dimStatusBar(boolean dim) {
         if (Util.isHoneycombOrLater()) {
             if (dim) {
-                mSurface.setSystemUiVisibility(Util.isICSOrLater()
+                mSurface.setSystemUiVisibility(Util.hasNavBar()
                         ? View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                         : View.SYSTEM_UI_FLAG_LOW_PROFILE);
             } else {
@@ -939,11 +1206,9 @@ public class VideoPlayerActivity extends Activity {
      */
     private void load() {
         mLocation = null;
-        String title = null;
-        String lastLocation = null;
-        long lastTime = 0;
+        String title = getResources().getString(R.string.title);
         boolean dontParse = false;
-        SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
+        String itemTitle = null;
 
         if (getIntent().getAction() != null
                 && getIntent().getAction().equals(Intent.ACTION_VIEW)) {
@@ -952,18 +1217,24 @@ public class VideoPlayerActivity extends Activity {
         } else if(getIntent().getExtras() != null) {
             /* Started from VideoListActivity */
             mLocation = getIntent().getExtras().getString("itemLocation");
+            itemTitle = getIntent().getExtras().getString("itemTitle");
             dontParse = getIntent().getExtras().getBoolean("dontParse");
         }
 
-        if (mLocation != null && mLocation.length() > 0 && !dontParse) {
-            mLibVLC.readMedia(mLocation, false);
-            mSurface.setKeepScreenOn(true);
+        mSurface.setKeepScreenOn(true);
 
-            // Save media for next time, and restore position if it's the same one as before
-            lastLocation = preferences.getString(PreferencesActivity.LAST_MEDIA, null);
-            lastTime = preferences.getLong(PreferencesActivity.LAST_TIME, 0);
-            if (lastTime > 0 && mLocation.equals(lastLocation))
-                mLibVLC.setTime(lastTime);
+        /* Start / resume playback */
+        if (savedIndexPosition > -1) {
+            mLibVLC.playIndex(savedIndexPosition);
+        } else if (mLocation != null && mLocation.length() > 0 && !dontParse) {
+            savedIndexPosition = mLibVLC.readMedia(mLocation, false);
+        }
+
+        if (mLocation != null && mLocation.length() > 0 && !dontParse) {
+            // restore last position
+            Media media = DatabaseManager.getInstance(this).getMedia(this, mLocation);
+            if (media != null && media.getTime() > 0)
+                mLibVLC.setTime(media.getTime());
 
             try {
                 title = URLDecoder.decode(mLocation, "UTF-8");
@@ -975,7 +1246,9 @@ public class VideoPlayerActivity extends Activity {
                 if (dotIndex != -1)
                     title = title.substring(0, dotIndex);
             }
-            mTitle.setText(title);
+        } else if(itemTitle != null) {
+            title = itemTitle;
         }
+        mTitle.setText(title);
     }
 }
